/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/



/*


This code is copied from the old version of gsl_mroot_hybrids_ts.cpp
but doesn't really work as a benchmark yet

*/


#include <cmath>
#include <o2scl/test_mgr.h>
#include <o2scl/mm_funct.h>
#include <o2scl/gsl_mroot_hybrids.h>

int gfn(size_t nv, const o2scl::ovector_view &x, 
	o2scl::ovector_view &y, void *&pa) {
  y[0]=sin(x[1]-0.2);
  y[1]=sin(x[0]-0.25);
  return 0;
}

class cl {
public:
  int mfn(size_t nv, const o2scl::ovector_view &x, o2scl::ovector_view &y, 
	  void *&pa) {
    y[0]=sin(x[1]-0.2);
    y[1]=sin(x[0]-0.25);
    return 0;
  }
  int mfnd(size_t nv, o2scl::ovector_view &x, o2scl::ovector_view &y,
	   o2scl::omatrix_view &j, void *&pa) {
    j[0][0]=0.0;
    j[0][1]=cos(x[1]-0.2);
    j[1][0]=cos(x[0]-0.25);
    j[1][1]=0.0;
    return 0;
  }

  int mfna(size_t nv, const double x[2], double y[2], void *&pa) {
    y[0]=sin(x[1]-0.2);
    y[1]=sin(x[0]-0.25);
    return 0;
  }
};

using namespace std;
using namespace o2scl;

int main(void) {
  cl acl;
  ovector x(2);
  double xa[2];
  int i;
  void *vp=NULL;
  size_t tmp;
  int N=1;
  int t1=0, t2=0, t3=0, t4=0, t1b=0;
  test_mgr t;

  t.set_output_level(2);


  for(int kk=0;kk<1;kk++) {

    // 1 - Non-templated access through a funct object 
    mm_funct_mfptr<cl,void *,ovector_view> fmf(&acl,&cl::mfn);
    gsl_mroot_hybrids<void *> cr1;
    
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	cr1.msolve(2,x,vp,fmf);
      }
    }
    t1+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"1a");
    t.test_rel(x[1],0.2,1.0e-6,"1b");

    // 1b
    jac_funct_mfptr<cl,void *,ovector_view,omatrix_view> fmfd(&acl,&cl::mfnd);
    
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	cr1.msolve_de(2,x,vp,fmf,fmfd);
      }
    }
    t1b+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"1a");
    t.test_rel(x[1],0.2,1.0e-6,"1b");

    // 2 - Non-templated access through a funct object 
    mm_vfunct_mfptr<cl,void *,2> fmf2(&acl,&cl::mfna);
    gsl_mroot_hybrids<void *,mm_vfunct_mfptr<cl,void *,2>,double[2],
      double[2],array_alloc<double[2]> > cr2;
    
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	xa[0]=0.5;
	xa[1]=0.5;
	cr2.msolve(2,xa,vp,fmf2);
      }
    }
    t2+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(xa[0],0.25,1.0e-6,"1a");
    t.test_rel(xa[1],0.2,1.0e-6,"1b");
    
    // 4 - Templated access through a global function pointer
    typedef int (*gfnt)(size_t, const ovector_view &, ovector_view &, 
			void *&);
    gsl_mroot_hybrids<void *,gfnt,ovector_view> cr4;
    gfnt gfnv=&gfn;

    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	cr4.msolve(2,x,vp,gfnv);
      }
    }
    t4+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"4a");
    t.test_rel(x[1],0.2,1.0e-6,"4b");
    cout << endl;
  }

  cout << t1 << endl;
  cout << t2 << endl;
  cout << t3 << endl;
  cout << t4 << endl;

  /*
    gsl_mroot_hybrids_gfn<void *,int> crx;
    a=1.0e-5;
    b=1.0;
    crx.solve_bkt(a,b,vp,i);
    t.test_rel(a,0.2,1.0e-6,"solve");
  */

  /*
    a=1.0e-5;
    b=1.0;
    crx.solve(a,vp,i);
    t.test_rel(a,0.2,1.0e-6,"solve");
  */
  
  t.report();
  return 0;
}
