#include <iostream>

#include <o2scl/funct.h>
#include <o2scl/table.h>
#include <o2scl/eqi_deriv.h>
#include <o2scl/interp.h>
#include <o2scl/base_ioc.h>

// For cout
using namespace std;
// Most of the library resides in this namespace
using namespace o2scl;

int main(void) {
  size_t bign=30, i;
  
  cout.precision(6);
  cout.setf(ios::scientific);

  ovector x(bign), y(bign), dydx(bign), dydx2(bign);

  // We create some data, y=exp(x).
  for(i=0;i<bign;i++) {
    x[i]=((double)i)/3.0;
    y[i]=exp(x[i]);
  }

  // Compute dydx using derivative formulas for equally-spaced abscissas
  eqi_deriv<void *,funct<void *> > ei;
  ei.deriv_array(bign,1.0/3.0,y,dydx);

  // Compute the derivative using the interpolation 
  // (natural cubic splines are the default)
  cspline_interp<ovector_view> it;
  o2scl_interp_vec<> gi(it,bign,x,y);
  for(i=0;i<bign;i++) {
    dydx2[i]=gi.deriv(x[i]);
  }

  // Create a data table 
  table ta(100);
  // Give the column names
  ta.line_of_names("x y dydx dydx2 err err2");

  cout << "x            y            dydx         dydx'"
       << "        err          err'" << endl;
  for(i=0;i<bign;i++) {
    cout << x[i] << " " << y[i] << " " << dydx[i] << " " << dydx2[i] << " ";
    cout << fabs(y[i]-dydx[i])/y[i] << " " << fabs(y[i]-dydx2[i])/y[i] << endl;
    // Add the data to the table as well...
    double line[6]={x[i],y[i],dydx[i],dydx2[i],
		    fabs(y[i]-dydx[i])/y[i],fabs(y[i]-dydx2[i])/y[i]};
    ta.line_of_data(6,line);
  }
  
  // Create a collection, the main I/O facility
  base_ioc bio;
  collection co;
  // Output results to file
  co.out_one("ex_diff.out","table","ex_diff",&ta);

  return 0;
}
  

