#include <o2scl/funct.h>
#include <o2scl/gsl_root_brent.h>

using namespace std;
using namespace o2scl;

class my_class {

private:

  double parameter;

public:
  
  void set_parameter() { parameter=0.1; }
  
  // A function demonstrating the different ways of implementing
  // function parameters
  double function_to_solve(double x, double &p) {
    return (-sin(x-0.2))/(fabs(x+0.01))+parameter+p;
  }
  
};

int main(void) {
  
  cout.setf(ios::scientific);
  
  // The solver, specifying the type of the parameter (double)
  // and the function type (funct<double>)
  gsl_root_brent<double,funct<double> > solver;

  my_class c;
  c.set_parameter();
  
  // This is the magic that allows specification of class member
  // functions as functions to solve. This object-oriented approach
  // avoids the use of static variables and functions and multiple
  // inheritance at the expense of a little overhead. We need to
  // provide the address of an instantiated object and the address of
  // the member function.
  funct_mfptr_noerr<my_class,double> function(&c,&my_class::function_to_solve);
  
  double x1=-10.0;
  double x2=1.4;
  double p=0.1;
  
  // The value verbose=1 prints out iteration information
  // and verbose=2 requires a keypress between iterations.
  // The parameter p=0.1 is used.
  solver.verbose=1;
  solver.solve_bkt(x1,x2,p,function);

  // This is actually a somewhat difficult function to solve because
  // of the sinusoidal behavior.
  cout << "Solution: " << x1 
       << " Function value: " << c.function_to_solve(x1,p) << endl;

  return 0;
}
