#include <o2scl/cern_root.h>
#include "ex_speed_lib.h"

using namespace std;
using namespace o2scl;

int old_solver(double &a0, double b0, void *pa, 
	       int (*func)(double x, double &y, void *vp));

int old_solver_lib(double &a0, double b0, void *pa, 
		   int (*func)(double x, double &y, void *vp));

class my_class {
public:
  double function_to_solve(double x, void *vp) {
    return (-sin(x-0.2))/(fabs(x+0.01))+0.2;
  }
};

class faster_method : public funct {
  int operator()(double x, double &y, void *pa) {
    y=(-sin(x-0.2))/(fabs(x+0.01))+0.2;
    return 0;
  }
};

int global_fptr(double x, double &y, void *pa) {
  y=(-sin(x-0.2))/(fabs(x+0.01))+0.2;
  return 0;
}

int main(void) {
  double x1, x2;
  int i, j;
  const int n=NITER;

  cout.setf(ios::scientific);

#ifdef MODE1
  cern_root solver;
  my_class c;
  funct_mfptr_noerr<my_class> function(&c,&my_class::function_to_solve);

  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
      
      solver.solve_bkt(x1,x2,NULL,function);
    }
  }
#endif

#ifdef MODE2
  cern_root solver;
  faster_method f;

  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
      
      solver.solve_bkt(x1,x2,NULL,f);
    }
  }
#endif

#ifdef MODE3
  cern_root solver;
  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
  
      solver.solve_bkt_fast(x1,x2,NULL,&global_fptr);
    }
  }
#endif

#ifdef MODE4
  newsolver ns;

  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
  
      ns.solve(x1,x2,NULL,&global_fptr);
    }
  }
#endif

#ifdef MODE5
  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
  
      old_solver_lib(x1,x2,NULL,&global_fptr);
    }
  }
#endif

#ifdef MODE6
  for(int i=0;i<n;i++) {
    for(int j=0;j<=n;j++) {
      x1=-10.0;
      x2=1.4;
  
      old_solver(x1,x2,NULL,&global_fptr);
    }
  }
#endif

  cout << "Solution: " << x1 << endl;
  
  return 0;
}

inline double sign(double a, double b) {
  if (b>=0.0) return fabs(a);
  return -fabs(a);
}

int old_solver(double &a0, double b0, void *pa,
	       int (*func)(double x, double &y, void *vp)) {
  
  double im1[2]={2.0,3.0}, im2[2]={-1.0,3.0}, c=0.0, fa, fb, atl, a, b, mft;
  double fc=0.0, d=0.0, fd=0.0, tol, h, hb, w, p, q, fdb, fda, f=0.0;
  bool lmt[2];
  int ie=0, loop, mf;
  char ch;
  int mode=1;
  double tolx=1.0e-8;
  int ntrial=200;

  func(a0,fb,pa);
  func(b0,fa,pa);
  
  if (fa*fb>0.0) {
    return 2;
  }
  
  atl=fabs(tolx);
  b=a0;
  a=b0;
  lmt[1]=true;
  mf=2;
  loop=1;
  do {
    if (loop==1) {
      c=a;
      fc=fa;
      ie=0;
    } else if (loop==2) {
      ie=0;
    }
    if (fabs(fc)<fabs(fb)) {
      if (c!=a) {
	d=a;
	fd=fa;
      }
      a=b;
      b=c;
      c=a;
      fa=fb;
      fb=fc;
      fc=fa;
    }
    tol=atl*(1.0+fabs(c));
    h=0.5*(c+b);
    hb=h-b;

    if (fabs(hb)>tol) {
      if (ie>im1[mode-1]) {
	w=hb;
      } else {
	tol*=sign(1.0,hb);
	p=(b-a)*fb;
	lmt[0]=(ie<=1);
	if (lmt[mode-1]) {
	  q=fa-fb;
	  lmt[1]=false;
	} else {
	  fdb=(fd-fb)/(d-b);
	  fda=(fd-fa)/(d-a);
	  p*=fda;
	  q=fdb*fa-fda*fb;
	}
	if (p<0.0) {
	  p=-p;
	  q=-q;
	}
	if (ie==im2[mode-1]) p+=p;
	if (p==0.0 || p<=q*tol) {
	  w=tol;
	} else if (p<hb*q) {
	  w=p/q;
	} else {
	  w=hb;
	}
      }
      d=a;
      a=b;
      fd=fa;
      fa=fb;
      b+=w;
      mf++;
      if (mf>ntrial) {
	return 3;
      }
      func(b,fb,pa);
      if (fb==0.0 || sign(1.0,fc)==sign(1.0,fb)) {
	loop=1;
      } else if (w==hb) {
	loop=2;
      } else {
	ie++;
	loop=3;
      }
    } else {
      loop=0;
    }
  } while (loop>0);
  a0=c;
  return 0;
}

