/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/table.h>
#include <o2scl/test_mgr.h>
#include <o2scl/binary_file.h>
#include <o2scl/base_ioc.h>

using namespace std;
using namespace o2scl;

int main(void) {
  table at(20), *at2, *at3;
  ofstream fout;
  int i;
  test_mgr t;
  t.set_output_level(2);
  collection co, *cop;

  cout.setf(ios::scientific);

  t.test_gen(at.new_column("col1")==0,"new_column() (1)");
  t.test_gen(at.new_column("col2")==0,"new_column() (2)");

  t.test_str(at.get_column_name(0),"col1","get_column_name() (1)");
  t.test_str(at.get_column_name(1),"col2","get_column_name() (2)");
  t.test_str(at.get_column_name(2),"","get_column_name() (3)");

  double line1[2]={1.0,2.5};
  double line2[2]={3.0,4.5};
  double line3[2]={5.0,3.5};
  at.line_of_data(2,line1);
  at.line_of_data(2,line2);
  at.line_of_data(2,line3);
  t.test_rel(at.get("col1",0),1.0,1.0e-14,"line_of_data(new) and get() (1)");
  t.test_rel(at.get("col2",at.get_nlines()-1),3.5,1.0e-14,
	     "line_of_data(new) and get() (2)");
  t.test_abs(at.get("colx",1),0.0,1.0e-14,"get() (1)");
  t.test_abs(at.get("col1",3),0.0,1.0e-14,"get() (3)");
  
  table_io_type table_io;
  t.test_gen(table_io.add(co,"at",&at)==0,"collection::add()");

  t.test_gen(co.npa()==1,"collection::npa() (1)");
  
  cout << "\nTesting print:" << endl;
  co.summary(&cout); 
  
  text_out_file tof(&cout);
  co.fout(&tof);

  cout << endl;
  
  t.test_rel(at["col1"][1],3.0,1.0e-6,"test operator[string]");

  t.report();

  return 0;
}


