#include <o2scl/cfl_njl_eos.h>

#include <o2scl/bag_eos.h>
#include <o2scl/eff_quark.h>
#include <o2scl/gsl_mroot_hybrids.h>
#include <o2scl/gsl_inte_qng.h>
#include <o2scl/gsl_mroot_hybrids.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  // Various initializations

  double x[3], qq1, qq2, qq3, y[3], rr1, rr2, rr3, ss1, ss2, ss3;
  int i;

  cout.setf(ios::scientific);

  nambujl_eos nj;
  nambujl_eos njt;
  cfl_njl_eos cfl;

  quark u(nj.up_default_mass,6.0);
  quark d(nj.down_default_mass,6.0);
  quark s(nj.strange_default_mass,6.0);
  quark u2(nj.up_default_mass,6.0);
  quark d2(nj.down_default_mass,6.0);
  quark s2(nj.strange_default_mass,6.0);
  quark u3(nj.up_default_mass,6.0);
  quark d3(nj.down_default_mass,6.0);
  quark s3(nj.strange_default_mass,6.0);
  
  gsl_mroot_hybrids<void *,mm_funct<void *> > nd;
  gsl_inte_qng<void *,funct<void *> > gl, gl2;
  thermo th, th2, th3;
  
  nd.tolf/=100.0;
  nd.tolx/=100.0;
  
  nj.set_quarks(u,d,s);
  nj.set_thermo(th);
  nj.set_parameters();

  njt.set_quarks(u2,d2,s2);
  njt.set_thermo(th2);
  njt.set_parameters();

  cfl.set_quarks(u3,d3,s3);
  cfl.set_thermo(th3);
  cfl.set_parameters();
  
  gsl_inte_qng<void *,funct<void *> > ngnew;
  cfl.set_inte(ngnew);
  
  mm_funct_mfptr<nambujl_eos,void *> fqq(&nj,&nambujl_eos::gapfunqq);
  mm_funct_mfptr<nambujl_eos,void *> fqq2(&njt,&nambujl_eos::gapfunqq);
  mm_funct_mfptr<nambujl_eos,void *> fqq3(&cfl,&nambujl_eos::gapfunqq);
  
  // Set the quark chemical potentials

  u.mu=2.5;
  d.mu=2.5;
  s.mu=2.5;
  u2.mu=2.5;
  d2.mu=2.5;
  s2.mu=2.5;
  
  // Compute the masses from the quark condensates by 
  // solving the gap equation
  
  x[0]=-1.0;
  x[1]=-1.0;
  x[2]=-2.0;
  ovector_array ax(3,x), ay(3,y);
  void *vpx=(void *)(&nj);
  
  nj.fromqq=true;
  nd.msolve(3,ax,vpx,fqq);
  cout << ax << endl;

  // Compute the quark condensates from the masses, 
  // and show the solution works
  
  x[0]=u.ms;
  x[1]=d.ms;
  x[2]=s.ms;
  nj.fromqq=false;
  nj.gapfunms(3,ax,ay,vpx);
  t.test_rel(y[0],0.0,5.0e-9,"zero T ungapped gap eqn 1");
  t.test_rel(y[1],0.0,5.0e-9,"zero T ungapped gap eqn 2");
  t.test_rel(y[2],0.0,1.0e-6,"zero T ungapped gap eqn 3");

  // Now compute the full EOS at T=0
  
  nj.calc_eq_p(u,d,s,qq1,qq2,qq3,th);

  cout << "Comparing nambujl_eos, nambujl_eos, and cfl_njl_eos:" 
       << endl;
  cout << "qq: " << u.qq << " " << d.qq << " " << s.qq << endl;
  cout << "sol: " << qq1 << " " << qq2 << " " << qq3 << endl;
  cout << "ms: " << u.ms << " " << d.ms << " " << s.ms << endl;
  cout << "n: " << u.n << " " << d.n << " " << s.n << endl;
  cout << "mu: " << u.mu << " " << d.mu << " " << s.mu << endl;
  cout << "ed: " << u.ed << " " << d.ed << " " << s.ed << endl;
  cout << "pr: " << u.pr << " " << d.pr << " " << s.pr << endl;
  cout << "th: " << th.ed << " " << th.pr << " " << th.en << endl;
  cout << endl;

  // Compute the masses from the quark condensates by 
  // solving the gap equation again for the 'njt' object

  x[0]=-1.0;
  x[1]=-1.0;
  x[2]=-2.0;
  void *vpy=(void *)(&njt);
  njt.fromqq=true;
  nd.msolve(3,ax,vpy,fqq2);
  cout << ax << endl;

  // Now compute the full EOS at T!=0

  u2.qq=x[0];
  d2.qq=x[1];
  s2.qq=x[2];
  njt.fromqq=true;
  njt.calc_eq_temp_p(u2,d2,s2,rr1,rr2,rr3,th2,0.01);
  
  cout << "qq: " << u2.qq << " " << d2.qq << " " << s2.qq << endl;
  cout << "sol: " << rr1 << " " << rr2 << " " << rr3 << endl;
  cout << "ms: " << u2.ms << " " << d2.ms << " " << s2.ms << endl;
  cout << "n: " << u2.n << " " << d2.n << " " << s2.n << endl;
  cout << "mu: " << u2.mu << " " << d2.mu << " " << s2.mu << endl;
  cout << "ed: " << u2.ed << " " << d2.ed << " " << s2.ed << endl;
  cout << "pr: " << u2.pr << " " << d2.pr << " " << s2.pr << endl;
  cout << "th: " << th2.ed << " " << th2.pr << " " << th2.en << endl;
  cout << endl;

  // Compare T=0 and T!=0 results

  t.test_rel(u2.qq,u.qq,1.0e-6,"qqu");
  t.test_rel(d2.qq,d.qq,1.0e-6,"qqd");
  t.test_rel(s2.qq,s.qq,1.0e-6,"qqs");
  t.test_rel(rr1,0.0,1.0e-3,"qq1");
  t.test_rel(rr2,0.0,1.0e-3,"qq2");
  t.test_rel(rr3,0.0,1.0e-1,"qq3");
  t.test_rel(u2.n,u.n,1.0e-2,"nu");
  t.test_rel(d2.n,d.n,1.0e-2,"nd");
  t.test_rel(s2.n,s.n,1.0e-1,"ns");
  t.test_rel(u2.mu,u.mu,1.0e-6,"muu");
  t.test_rel(d2.mu,d.mu,1.0e-6,"mud");
  t.test_rel(s2.mu,s.mu,1.0e-6,"mus");
  t.test_rel(u2.ed,u.ed,1.0e-2,"edu");
  t.test_rel(d2.ed,d.ed,1.0e-2,"edd");
  t.test_rel(s2.ed,s.ed,1.0e-2,"eds");
  t.test_rel(u2.pr,u.pr,1.0e-4,"pru");
  t.test_rel(d2.pr,d.pr,1.0e-4,"prd");
  t.test_rel(s2.pr,s.pr,1.0e-4,"prs");
  t.test_rel(th.ed,th2.ed,1.0e-1,"thed");
  t.test_rel(th.pr,th2.pr,5.0e-3,"thpr");
  t.test_rel(th2.en,th.en,2.0e-1,"then");

  // Now compare the Delta=0 code with the NJL model at T!=0

  double gap1, gap2, gap3, n3, n8;
  cern_quartic_real_coeff cq;

  u3.mu=2.5;
  d3.mu=2.5;
  s3.mu=2.5;
  u3.del=0.0;
  d3.del=0.0;
  s3.del=0.0;

  cfl.gap_limit=-0.01;

  // Solve the gap equations
  x[0]=-1.0;
  x[1]=-1.0;
  x[2]=-2.0;
  void *vpz=(void *)(&cfl);
  nd.msolve(3,ax,vpz,fqq3);
  cout << ax << endl;
  
  cfl.calc_eq_temp_p(u3,d3,s3,ss1,ss2,ss3,gap1,gap2,gap3,
		  0.0,0.0,n3,n8,th3,0.01);
  
  cout << "qq: " << u3.qq << " " << d3.qq << " " << s3.qq << endl;
  cout << "sol: " << ss1 << " " << ss2 << " " << ss3 << endl;
  cout << "ms: " << u3.ms << " " << d3.ms << " " << s3.ms << endl;
  cout << "n: " << u3.n << " " << d3.n << " " << s3.n << endl;
  cout << "mu: " << u3.mu << " " << d3.mu << " " << s3.mu << endl;
  cout << "ed: " << u3.ed << " " << d3.ed << " " << s3.ed << endl;
  cout << "pr: " << u3.pr << " " << d3.pr << " " << s3.pr << endl;
  cout << "th: " << th3.ed << " " << th3.pr << " " << th3.en << endl;

  t.test_rel(u2.qq,u3.qq,1.0e-6,"qqu");
  t.test_rel(d2.qq,d3.qq,1.0e-6,"qqd");
  t.test_rel(s2.qq,s3.qq,1.0e-6,"qqs");
  t.test_rel(ss1,0.0,1.0e-3,"qq1");
  t.test_rel(ss2,0.0,1.0e-3,"qq2");
  t.test_rel(ss3,0.0,1.0e-2,"qq3");
  t.test_rel(u2.n,u3.n,1.0e-2,"nu");
  t.test_rel(d2.n,d3.n,1.0e-2,"nd");
  t.test_rel(s2.n,s3.n,1.0e-1,"ns");
  t.test_rel(u2.mu,u3.mu,1.0e-6,"muu");
  t.test_rel(d2.mu,d3.mu,1.0e-6,"mud");
  t.test_rel(s2.mu,s3.mu,1.0e-6,"mus");
  t.test_rel(u2.ed,u3.ed,1.0e-2,"edu");
  t.test_rel(d2.ed,d3.ed,1.0e-2,"edd");
  t.test_rel(s2.ed,s3.ed,1.0e-2,"eds");
  t.test_rel(u2.pr,u3.pr,1.0e-4,"pru");
  t.test_rel(d2.pr,d3.pr,1.0e-4,"prd");
  t.test_rel(s2.pr,s3.pr,1.0e-4,"prs");
  t.test_rel(th3.ed,th2.ed,5.0e-2,"thed");
  t.test_rel(th3.pr,th2.pr,6.0e-4,"thpr");
  t.test_rel(th2.en,th3.en,1.0e-1,"then");

  t.test_rel(th3.pr,1.423674,1.0e-3,"make sure we're not changing.");
  //t.test_rel(th3.en,0.08216656,4.0e-3,"make sure we're not changing2.");

  cout << "\nTest thd_potential(): " 
       << cfl.thd_potential(u3,d3,s3,0.0,0.0,0.01) << endl;
  t.test_rel(-cfl.thd_potential(u3,d3,s3,0.0,0.0,0.01),
	     th3.pr,1.0e-6,"thd_potential:pr");
  
  ovector egv(36);
  gsl_vector *ev=(gsl_vector *)egv.get_gsl_vector();
  ovector fgv(36);
  gsl_vector *fv=(gsl_vector *)fgv.get_gsl_vector();

  // Test CFL phase
  u3.mu=2.5;
  d3.mu=2.5;
  s3.mu=2.5;
  u3.ms=0.0;
  d3.ms=0.0;
  s3.ms=0.0;
  u3.del=0.5;
  d3.del=0.5;
  s3.del=0.5;
  cfl.eigenvalues(1.0,egv,0.0,0.0);
  
  fgv[0]=sqrt(pow(1.0+2.5,2.0)+4.0*0.5*0.5);
  fgv[1]=sqrt(pow(1.0-2.5,2.0)+4.0*0.5*0.5);
  fgv[2]=-sqrt(pow(1.0-2.5,2.0)+4.0*0.5*0.5);
  fgv[3]=-sqrt(pow(1.0+2.5,2.0)+4.0*0.5*0.5);
  for(i=0;i<8;i++) {
    fgv[4+4*i]=sqrt(pow(1.0+2.5,2.0)+0.5*0.5);
    fgv[5+4*i]=sqrt(pow(1.0-2.5,2.0)+0.5*0.5);
    fgv[6+4*i]=-sqrt(pow(1.0-2.5,2.0)+0.5*0.5);
    fgv[7+4*i]=-sqrt(pow(1.0+2.5,2.0)+0.5*0.5);
  }
  
  gsl_sort_vector(ev);
  gsl_sort_vector(fv);
  cout << "\nTest eigenvalues of CFL phase: " << endl;
  for(i=0;i<36;i++) {
    t.test_rel(gsl_vector_get(ev,i),gsl_vector_get(fv,i),1.0e-6,"CFL");
  }

  //Test 2SC phase 
  u3.mu=2.5;
  d3.mu=2.5;
  s3.mu=2.5;
  u3.ms=0.0;
  d3.ms=0.0;
  s3.ms=1.5;
  u3.del=0.0;
  d3.del=0.0;
  s3.del=0.5;
  cfl.eigenvalues(1.0,egv,0.0,0.0);
  
  for(i=0;i<3;i++) {
    fgv[0+4*i]=2.5-sqrt(1.0*1.0+1.5*1.5);
    fgv[1+4*i]=2.5+sqrt(1.0*1.0+1.5*1.5);
    fgv[2+4*i]=-2.5-sqrt(1.0*1.0+1.5*1.5);
    fgv[3+4*i]=-2.5+sqrt(1.0*1.0+1.5*1.5);
  }
  for(i=0;i<2;i++) {
    fgv[12+4*i]=-1.0+2.5;
    fgv[13+4*i]=-1.0-2.5;
    fgv[14+4*i]=1.0+2.5;
    fgv[15+4*i]=1.0-2.5;
  }
  for(i=0;i<4;i++) {
    fgv[20+4*i]=sqrt(pow(1.0+2.5,2.0)+0.5*0.5);
    fgv[21+4*i]=sqrt(pow(1.0-2.5,2.0)+0.5*0.5);
    fgv[22+4*i]=-sqrt(pow(1.0-2.5,2.0)+0.5*0.5);
    fgv[23+4*i]=-sqrt(pow(1.0+2.5,2.0)+0.5*0.5);
  }
  
  gsl_sort_vector(ev);
  gsl_sort_vector(fv);

  cout << "\nTest eigenvalues of 2SC phase: " << endl;
  for(i=0;i<36;i++) {
    t.test_rel(gsl_vector_get(ev,i),gsl_vector_get(fv,i),1.0e-6,"2SC");
  }

  cout << "\nCompare fixed mass and zero gaps with bag model:" << endl;
  bag_eos bag;
  bag.bag_constant=0.0;

  u3.del=0.0;
  d3.del=0.0;
  s3.del=0.0;
  cfl.gap_limit=-1.0;
  cfl.fixed_mass=true;
  cfl.calc_eq_temp_p(u3,d3,s3,ss1,ss2,ss3,gap1,gap2,gap3,
		  0.0,0.0,n3,n8,th3,0.1);

  eff_quark u4(u.m,6.0), d4(d.m,6.0), s4(s.m,6.0);
  u4.mu=u.mu;
  d4.mu=d.mu;
  s4.mu=s.mu;
  thermo th4;

  int ret=bag.calc_temp_p(u4,d4,s4,0.1,th4);
  t.test_gen(ret==0,"bag_temp success");
  t.test_rel(th4.ed-th3.ed,th3.pr-th4.pr,1.0e-2,"bag constant is constant");

  t.test_rel(u3.n,u4.n,1.0e-2,"nu");
  t.test_rel(d3.n,d4.n,1.0e-2,"nd");
  t.test_rel(s3.n,s4.n,1.0e-2,"ns");
  t.test_rel(th3.en,th4.en,4.0e-2,"entropy");

  cout << endl;
  ovector eigen(36), eigen2(36);
  u3.mu=3.0;
  d3.mu=3.0;
  s3.mu=3.0;
  u3.del=0.5;
  d3.del=0.5;
  s3.del=0.5;
  u3.ms=0.0;
  d3.ms=0.0;
  s3.ms=0.0;
  
  cfl.eigenvalues(1.0,eigen,0.01,0.0);
  cfl.faster=true;
  cfl.eigenvalues(1.0,eigen2,0.01,0.0);
  cfl.faster=false;
  
  gsl_vector *gv1=(gsl_vector *)eigen.get_gsl_vector();
  gsl_sort_vector(gv1);
  gsl_vector *gv2=(gsl_vector *)eigen2.get_gsl_vector();
  gsl_sort_vector(gv2);

  for(int l=0;l<36;l++) {
    cout << l << " " 
	 << gsl_vector_get(gv1,l) << " " 
	 << gsl_vector_get(gv2,l) << endl;
    t.test_rel(gsl_vector_get(gv1,l),gsl_vector_get(gv2,l),5.0e-3,"fast");
  }

  cout << endl;

  t.report();

  // Test new version of calc_temp_p().
  cfl.zerot=true;
  cfl.fixed_mass=false;
  cfl.color_neut=true;
  cfl.gap_limit=-0.01;
  cfl.set_parameters();
  u.del=0.5;
  d.del=0.5;
  s.del=0.5;
  u.qq=-0.05;
  d.qq=-0.05;
  s.qq=-0.5;

  thermo thx;

  double muq=2.7;
  int ret2;
  
  u.mu=muq;
  d.mu=muq;
  s.mu=muq;
  ret2=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
  cout << (u.n+d.n+s.n)/3.0 << " " << ret2 << endl;
  cout << u.qq << " " << d.qq << " " << s.qq << endl;
  cout << u.del << " " << d.del << " " << s.del << endl;

  t.test_gen(ret2==0,"err val");
  t.test_rel(u.qq,-0.03406,1.0e-4,"qqu");
  t.test_rel(d.qq,-0.03406,1.0e-4,"qqd");
  t.test_rel(s.qq,-0.31856,1.0e-4,"qqs");
  t.test_rel(u.del,0.38061,1.0e-4,"delu");
  t.test_rel(d.del,0.38061,1.0e-4,"deld");
  t.test_rel(s.del,0.38728,1.0e-4,"dels");
  
  cfl.use_solver=false;
  ret2=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
  cout << (u.n+d.n+s.n)/3.0 << " " << ret2 << endl;
  cout << u.qq << " " << d.qq << " " << s.qq << endl;
  cout << u.del << " " << d.del << " " << s.del << endl;

  t.test_gen(ret2==0,"err val");
  t.test_rel(u.qq,-0.03406,5.0e-3,"qqu");
  t.test_rel(d.qq,-0.03406,5.0e-3,"qqd");
  t.test_rel(s.qq,-0.31856,5.0e-3,"qqs");
  t.test_rel(u.del,0.38061,5.0e-3,"delu");
  t.test_rel(d.del,0.38061,5.0e-3,"deld");
  t.test_rel(s.del,0.38728,5.0e-3,"dels");
  
  if (false) {
    cfl.GD*=1.5;
    
    for(muq=2.7;muq>1.7;muq-=0.05) {
      u.mu=muq;
      d.mu=muq;
      s.mu=muq;
      int ret3=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
      cout << (u.n+d.n+s.n)/3.0 << " " << ret3 << endl;
      if (ret3!=0) cout << err_hnd->get_str() << endl;
      cout << u.qq << " " << d.qq << " " << s.qq << endl;
      cout << muq << " " << u.del << " " << d.del << " " << s.del << " " 
	   << cfl.solver_qual << endl;
    }
  }

  t.report();

  return 0;
}
  

