/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/had_gibbs_nseos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

#include <o2scl/test_mgr.h>
#include <o2scl/gen_potential_eos.h>
#include <o2scl/schematic_eos.h>

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);
  cout.precision(5);

  // The neutrons and protons, thermo objects
  nonrel_fermion n1(o2scl_fm::mass_neutron,2.0);
  nonrel_fermion p1(o2scl_fm::mass_proton,2.0);
  n1.non_interacting=false;
  p1.non_interacting=false;
  //  fermion n2(o2scl_fm::mass_neutron,2.0);
  //  fermion p2(o2scl_fm::mass_proton,2.0);
  nonrel_fermion n2(o2scl_fm::mass_neutron,2.0);
  nonrel_fermion p2(o2scl_fm::mass_proton,2.0);
  n2.non_interacting=false;
  p2.non_interacting=false;
  thermo th1, th2;

  // The first EOS
  gen_potential_eos go;
  go.B=106.35/hc_mev_fm;
  go.sigma=4.0/3.0;
  go.rho0=0.16;
  go.Cu=-103.40/hc_mev_fm;
  go.Cl=-11.70/hc_mev_fm;
  go.form=go.mdi_form;
  go.Lambda=cbrt(1.5*pi2*go.rho0);
  go.x=0.0;
  go.Au=-95.98/hc_mev_fm-2.0*go.B*go.x/(go.sigma+1.0);
  go.Al=-120.75/hc_mev_fm+2.0*go.B*go.x/(go.sigma+1.0);

  // The second EOS
  schematic_eos se;
  se.eoa=-16.0/hc_mev_fm;
  se.comp=200.0/hc_mev_fm;
  se.a=20.0/hc_mev_fm;
  se.b=25.0/hc_mev_fm;
  se.kprime=0.0/hc_mev_fm;
  se.n0=0.16;

  /*

  // Setup
  had_gibbs_nseos hg;
  hg.set_hadronic_eos1(go);
  hg.set_hadronic_eos2(se);
  
  double chi, nb;
  int ret;

  chi=0.0;
  n1.n=0.3;
  p1.n=0.03;
  n2.n=0.3;
  p2.n=0.03;
  nb=0.3;
  
  se.n0=0.09;
  se.eoa=-10.0/hc_mev_fm;
  se.comp=20.0/hc_mev_fm;
  se.a=5.0/hc_mev_fm;
  se.b=10.718/hc_mev_fm;
  se.kprime=0.0;

  // Show that we have a beginning for the mixed phase
  ret=hg.fixed_chi(chi,n1,p1,th1,n2,p2,th2,nb);
  cout << n1.n << " " << p1.n << " " << n2.n << " " << p2.n << endl;
  cout << ret << " " << nb << endl;
  t.test_rel(nb,0.403177,1.0e-4,"beg mp");
  t.test_gen(ret==0,"Error 1");

  // Show that calc_e works
  hg.phase_guess=hg.mixed_phase;
  ret=hg.calc_e(nb+0.001,n1,p1,th1,n2,p2,th2,chi);
  cout << ret << " " << chi << endl;
  t.test_rel(chi,0.0364134,1.0e-4,"chi1");
  t.test_gen(ret==0,"Error 2");
  ret=hg.calc_e(nb+0.005,n1,p1,th1,n2,p2,th2,chi);
  cout << ret << " " << chi << endl;
  t.test_gen(ret==0,"Error 3");
  ret=hg.calc_e(nb+0.01,n1,p1,th1,n2,p2,th2,chi);
  cout << ret << " " << chi << endl;
  t.test_gen(ret==0,"Error 4");

  // Now try full version
  //  nd.verbose=2;
  hg.phase_guess=hg.phase1;

  n1.n=0.362;
  p1.n=0.0410;
  n2.n=0.389;
  p2.n=0.0424;

  for (nb=0.38;nb<=0.4501;nb+=0.01) {
  ret=hg.calc_e(nb,n1,p1,th1,n2,p2,th2,chi);
  cout << ret << " " << nb << " " << n1.n << " " << p1.n << " "
  << n2.n << " " << p2.n << " " << chi << " "
  << hg.phase_guess << endl;
  t.test_gen(ret==0,"Error 5");
  if (fabs(nb-0.042)<1.0e-6) {
  t.test_gen(hg.phase_guess==3,"Mixed phase");
  t.test_rel(chi,0.062030,1.0e-4,"chi mp");
  }
  }

  */

  t.report();
  return 0;
}

