/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_INTE_H
#define O2SCL_GSL_INTE_H

#include <gsl/gsl_machine.h>

#ifndef DOXYGENP
namespace o2scl {
#endif
  
  /** 
      \brief GSL integration base

      This base class does not perform any actual integration. 

      \todo Move the documentation below to a more sensible place
      
      If the function integ() uses \c tolx and \c tolf 
      to attempt to ensure that
      \f[ 
      |\mathrm{result}-I| \leq \mathrm{Max}(\mathrm{tolx},
      \mathrm{tolf}|I|)
      \f]
      and returns a value \c abserr to attempt to ensure that
      \f[
      |\mathrm{result}-I| \leq \mathrm{abserr} \leq
      \mathrm{Max}(\mathrm{tolx},\mathrm{tolf}|I|)
      \f]
      where \c I  is the integral to be evaluated. Even when 
      integ() returns success, these inequalities will fail for 
      some functions.
  */
  class gsl_inte {
    
  public:

    gsl_inte() {
    }
    
  protected:

    /// Desc
    double rescale_error(double err, const double result_abs, 
			 const double result_asc) {
      err = fabs(err);
      
      if (result_asc != 0 && err != 0) {
	
	double scale = pow((200 * err / result_asc), 1.5);
	
	if (scale < 1) {
	  err = result_asc * scale;
	} else {
	  err = result_asc;
	}
      }

      if (result_abs > GSL_DBL_MIN / (50 * GSL_DBL_EPSILON)) {

	double min_err = 50 * GSL_DBL_EPSILON * result_abs;
	
	if (min_err > err) {
	  err = min_err;
	}
      }
      
      return err;
    }
    
  };
  
  
#ifndef DOXYGENP
}
#endif

#endif
