/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_VEGAS_H
#define O2SCL_GSL_VEGAS_H

#include <iostream>
#include <o2scl/collection.h>
#include <o2scl/mcarlo_inte.h>
#include <o2scl/gsl_rnga.h>
#include <gsl/gsl_monte.h>
#include <gsl/gsl_monte_vegas.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Multidimensional integration using plain Vegas Carlo (GSL)

  */
  template<class param_t, class func_t, class rng_t=gsl_rnga, 
    class vec_t=ovector_view> class gsl_vegas : 
    public mcarlo_inte<param_t,func_t,rng_t,vec_t> {

      public:

      virtual ~gsl_vegas() {}
  
      /// Integrate function \c func from x=a to x=b.
      virtual int minteg_err(func_t &func, size_t ndim, 
			     const vec_t &a, const vec_t &b, param_t &pa,
			     double &res, double &err) 
      {
	gm_parms gmp={&func,(void *)&pa};
	gsl_monte_function gfunc={gsl_func,ndim,&gmp};

	int ncalls=this->n_points;

	double *aa=new double[ndim], *bb=new double[ndim];
	for(size_t i=0;i<ndim;i++) {
	  aa[i]=a[i];
	  bb[i]=b[i];
	}
  
	gsl_monte_vegas_state *s=gsl_monte_vegas_alloc(ndim);
	gsl_monte_vegas_integrate(&gfunc,aa,bb,ndim,ncalls,
				  this->def_rng.get_gsl_rng(),s,&res,&err);
	gsl_monte_vegas_free(s);
	
	delete[] aa;
	delete[] bb;

	return 0;

      }
      
      /// Return string denoting type ("gsl_vegas")
      virtual const char *type() { return "gsl_vegas"; }
      
      protected:
      
#ifndef DOXYGEN_INTERNAL
      
      /// The GSL function pointer
      static double gsl_func(double *x, size_t dim, void *pa) {
	gm_parms *gp=(gm_parms *)pa;
	ovector_array aa(dim,x);
	return (*gp->func)(dim,aa,*((param_t *)(gp->vp)));
      }
      
      /// The parameters for the GSL function pointer
      typedef struct gm_parms_s {
	func_t *func;
	void *vp;
      } gm_parms;
      
#endif
      
    };
  
#ifndef DOXYGENP
}
#endif

#endif

