/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_mmin_conf.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

double minfun(size_t n, const ovector_view &x, void *&pa) {
  double ret;
  ret=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return(ret);
}

int minfund(size_t n, ovector_view &x, ovector_view &g, void *&pa) {
  double ret;
  g[0]=2.0*x[0];
  g[1]=2.0*(x[1]-2.0);
  return 0;
}

int minfun2(size_t n, const double x[2], double &y, void *&pa) {
  y=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return 0;
}

int minfund2(size_t n, double x[2], double g[2], void *&pa) {
  double ret;
  g[0]=2.0*x[0];
  g[1]=2.0*(x[1]-2.0);
  return 0;
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  double min;
  ovector x(2);
  double xa[2];
  void *vp=0;
  int ret;
  gsl_mmin_conf<void *> g;
  gsl_mmin_conf_array<void *,2> g2;

  cout.setf(ios::scientific);

  multi_funct_fptr_noerr<void *> mf(minfun);
  grad_funct_fptr<void *> mfd(minfund);
  multi_vfunct_fptr<void *,2> mf2(minfun2);
  grad_vfunct_fptr<void *,2> mfd2(minfund2);
  
  x[0]=1.0;
  x[1]=1.0;
  ret=g.mmin(2,x,min,vp,mf);
  cout << ret << " " << min << " " << x[0] << " " << x[1] << endl;
  cout << err_hnd->get_str() << endl;
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_conf 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_conf 2");

  x[0]=1.0;
  x[1]=1.0;
  ret=g.mmin_de(2,x,min,vp,mf,mfd);
  cout << ret << " " << min << " " << x[0] << " " << x[1] << endl;
  cout << err_hnd->get_str() << endl;
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_conf 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_conf 2");

  xa[0]=1.0;
  xa[1]=1.0;
  ret=g2.mmin(2,xa,min,vp,mf2);
  cout << ret << " " << min << " " << xa[0] << " " << xa[1] << endl;
  cout << err_hnd->get_str() << endl;
  t.test_abs(xa[0],0.0,1.0e-4,"gsl_mmin_conf 1");
  t.test_rel(xa[1],2.0,2.0e-4,"gsl_mmin_conf 2");

  xa[0]=1.0;
  xa[1]=1.0;
  ret=g2.mmin_de(2,xa,min,vp,mf2,mfd2);
  cout << ret << " " << min << " " << xa[0] << " " << xa[1] << endl;
  cout << err_hnd->get_str() << endl;
  t.test_abs(xa[0],0.0,1.0e-4,"gsl_mmin_conf 1");
  t.test_rel(xa[1],2.0,2.0e-4,"gsl_mmin_conf 2");

  t.report();
  return 0;
}
 
