/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2scl/test_mgr.h>
#include <o2scl/mass_fit.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;

mnms95_mass mm;
mnms95_mass_exp mexp;
ame_mass ame;
full_dist fd(&mexp);
full_dist fd2(&ame);

double moller_comp() {
  
  double y=0.0;
  size_t n=0;
  for(nuclear_dist::iterator ndi=fd.begin();ndi!=fd.end();ndi++) {
    int Z=ndi->Z;
    int N=ndi->N;
    if (N>=8 && Z>=8) {
      y+=pow(mm.mass_excess_d(Z,N)-
	     mexp.mass_excess_d(Z,N),2.0);
      n++;
    }
  }
  y=sqrt(y/n);
  
  return y;
}

double moller_comp2() {
  
  double y=0.0;
  size_t n=0;
  for(nuclear_dist::iterator ndi=fd2.begin();ndi!=fd2.end();ndi++) {
    int Z=ndi->Z;
    int N=ndi->N;
    if (N>=8 && Z>=8) {
      y+=pow(mm.mass_excess_d(Z,N)-
	     ame.mass_excess_d(Z,N),2.0);
      n++;
    }
  }
  y=sqrt(y/n);
  
  return y;
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  cout.setf(ios::scientific);

  double res;
  semi_empirical_mass sem;
  mass_fit mf;
  mf.fit(sem,res);
  cout << sem.B << " " << sem.Sv << " " << sem.Ss << " " 
       << sem.Ec << " " << sem.Epair << endl;
  cout << res << endl;
  t.test_gen(res<4.0,"Successful fit.");
  
  res=moller_comp();
  cout << res << endl;
  t.test_rel(res,0.7,0.1,"Moller fit 1");

  res=moller_comp2();
  cout << res << endl;
  t.test_rel(res,0.88,0.1,"Moller fit 2");

  t.report();
  return 0;
}
  
