/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <iostream>
#include <o2scl/contour.h>
#include <o2scl/ovector_tlate.h>

using namespace std;
using namespace o2scl;

double fun(double x, double y) {
  return 15.0*exp(-pow(x-20.0,2.0)/400.0-pow(y-5.0,2.0)/25.0)
    +40.0*exp(-pow(x-70.0,2.0)/4900.0-pow(y-2.0,2.0)/4.0);
}

double fun2(double x, double y) {
  return 2.0*pow(x-4.0,2.0)+6.0*pow(y-2.0,2.0);
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);
  ofstream fout("contour.dat");
  
  // Initialization

  ovector x(10), y(10), xs(5), ys(5);
  double **data, **datas;
  int i, j, nsx, nsy, k;
  data=new double *[10];
  for(i=0;i<10;i++) data[i]=new double[10];
  datas=new double *[5];
  for(i=0;i<5;i++) datas[i]=new double[5];

  contour co;

  // Create initial array
  
  for(i=0;i<5;i++) {
    ys[i]=((double)i*2);
    xs[i]=ys[i]*ys[i];
  }

  for(j=0;j<5;j++) {
    for(i=0;i<5;i++) {
      datas[i][j]=15.0*exp(-pow(xs[i]-20.0,2.0)/400.0-
			   pow(ys[j]-5.0,2.0)/25.0);
      datas[i][j]+=40.0*exp(-pow(xs[i]-70.0,2.0)/4900.0-
			    pow(ys[j]-2.0,2.0)/4.0);
    }
  }

  co.set_data(5,5,xs,ys,datas);

  // Regrid
  
  co.regrid_data(2,2);
  
  size_t ngx=0, ngy=0;
  const ovector *xg=0, *yg=0;
  const ovector **datag=0;
  co.get_data(ngx,ngy,xg,yg,datag);
  
  // Now test contour lines

  for(i=0;i<10;i++) {
    y[i]=((double)i);
    x[i]=y[i]*y[i];
  }
  
  for(j=0;j<10;j++) {
    for(k=0;k<10;k++) {
      data[k][j]=fun(x[j],y[k]);
    }
  }
  
  cout << "Square 1:" << endl;

  ovector levels(7);
  levels[0]=5.0;
  levels[1]=10.0;
  levels[2]=15.0;
  levels[3]=20.0;
  levels[4]=25.0;
  levels[5]=30.0;
  levels[6]=35.0;
  
  int nc;

  co.set_data(10,10,x,y,data);
  co.set_levels(7,levels);
  double newcont[7];
  nc=co.calc_contours(newcont);

  fout << nc << endl;
  for(i=0;i<nc;i++) {
    ovector xt, yt;
    double val=0;
    int cs=0;
    xt.allocate(co.get_contour_size(i));
    yt.allocate(co.get_contour_size(i));
    co.get_contour(i,val,cs,xt,yt);
    fout << co.get_contour_size(i) << " ";
    for(j=0;j<cs;j++) {
      fout << xt[j] << " " << yt[j] << " ";
      t.test_rel(fun(xt[j],yt[j]),val,1.5e-1,"curve");
    }
    fout << endl;
  }
  
  // ------------------------------------------------------------

  cout << "Non-square 1:" << endl;

  // Test non-square data
  ovector sqx(10), sqy(8), sqlev(4);
  omatrix sqd(8,10);

  for(i=0;i<10;i++) sqx[i]=i*(8.0/9.0);
  for(i=0;i<8;i++) sqy[i]=i*(3.0/7.0);
  for(i=0;i<8;i++) {
    for(j=0;j<10;j++) {
      sqd[i][j]=2.0*pow(sqx[j]-4.0,2.0)+6.0*pow(sqy[i]-2.0,2.0);
    }
  }
  sqlev[0]=4.0;
  sqlev[1]=10.0;
  sqlev[2]=20.0;
  sqlev[3]=40.0;

  co.set_data(10,8,sqx,sqy,sqd);
  co.set_levels(4,sqlev);
  double newcont2[4];
  nc=co.calc_contours(newcont2);

  fout << nc << endl;
  for(i=0;i<nc;i++) {
    ovector xt, yt;
    double val=0;
    int cs=0;
    xt.allocate(co.get_contour_size(i));
    yt.allocate(co.get_contour_size(i));
    co.get_contour(i,val,cs,xt,yt);
    fout << co.get_contour_size(i) << " ";
    for(j=0;j<cs;j++) {
      fout << xt[j] << " " << yt[j] << " ";
      t.test_rel(fun2(xt[j],yt[j]),val,1.5e-1,"curve");
    }
  fout << endl;
  }
  
  // ------------------------------------------------------------

  cout << "Non-square 2:" << endl;
    
  // Test non-square data
  ovector srx(8), sry(10), srlev(4);
  omatrix srd(10,8);
    
  for(i=0;i<8;i++) srx[i]=i*(8.0/7.0);
  for(i=0;i<10;i++) sry[i]=i*(3.0/9.0);
  for(i=0;i<10;i++) {
    for(j=0;j<8;j++) {
      srd[i][j]=2.0*pow(srx[j]-4.0,2.0)+6.0*pow(sry[i]-2.0,2.0);
    }
  }
  srlev[0]=4.0;
  srlev[1]=10.0;
  srlev[2]=20.0;
  srlev[3]=40.0;
    
  co.set_data(8,10,srx,sry,srd);
  co.set_levels(4,srlev);
  double newcont3[4];
  co.verbose=1;
  nc=co.calc_contours(newcont3);
    
  fout << nc << endl;
  for(i=0;i<nc;i++) {
    ovector xt, yt;
    double val=0;
    int cs=0;
    xt.allocate(co.get_contour_size(i));
    yt.allocate(co.get_contour_size(i));
    co.get_contour(i,val,cs,xt,yt);
    fout << co.get_contour_size(i) << " ";
    for(j=0;j<cs;j++) {
      fout << xt[j] << " " << yt[j] << " ";
      t.test_rel(fun2(xt[j],yt[j]),val,1.7e-1,"curve");
    }
    fout << endl;
  }
  co.verbose=0;
  
  // ------------------------------------------------------------

  cout << "Contour with corners:" << endl;
  double cordat[7][7]={{1,2,3,4,5,6,7},
		       {0,1,2,3,4,5,6},
		       {1,0,1,2,3,4,5},
		       {2,1,0,1,2,3,4},
		       {3,2,1,0,1,2,3},
		       {4,3,2,1,0,1,2},
		       {5,4,3,2,1,0,1}};
  double corx[7]={0,1,2,3,4,5,6};
  double cory[7]={0,1,2,3,4,5,6};
  double corlev[4]={1,2,3,4};
  co.set_data(7,7,corx,cory,cordat);
  co.set_levels(4,corlev);
  double newcont4[4];
  nc=co.calc_contours(newcont4);
  cout.precision(10);
  cout << newcont4[0] << " "
       << newcont4[1] << " "
       << newcont4[2] << " "
       << newcont4[3] << endl;

  fout << nc << endl;
  for(i=0;i<nc;i++) {
    ovector xt, yt;
    double val=0;
    int cs=0;
    xt.allocate(co.get_contour_size(i));
    yt.allocate(co.get_contour_size(i));
    co.verbose=1;
    co.get_contour(i,val,cs,xt,yt);
    fout << co.get_contour_size(i) << " ";
    for(j=0;j<cs;j++) {
      if (i==0) cout << xt[j] << " " << yt[j] << endl;
      fout << xt[j] << " " << yt[j] << " ";
    }
    fout << endl;
  }
  
  fout.close();

  t.report();
  return 0;
}
