/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/twod_intp.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  int i,j;
  ovector x(3), y(3);
  omatrix data(3,3);

  cout.setf(ios::scientific);
  x[0]=0.0;
  x[1]=1.0;
  x[2]=2.0;
  y[0]=3.0;
  y[1]=2.0;
  y[2]=1.0;
  cout << "             | ";
  for(i=0;i<3;i++) cout << x[i] << " ";
  cout << endl;
  cout << "-------------|-";
  for(i=0;i<3;i++) cout << "-------------";
  cout << endl;
  for(i=0;i<3;i++) {
    cout << y[i] << " | ";
    for(j=0;j<3;j++) {
      data[i][j]=pow(sin(0.1*x[j]+0.3*y[i]),2.0);
      cout << data[i][j] << " ";
    }
    cout << endl;
  }
  cout << endl;

  cout << "Calc.        Exact" << endl;
  twod_intp ti;
  ti.set_data(3,3,x,y,data);
  cout << ti.interp(0.5,1.5) << " " 
       << pow(sin(0.1*0.5+0.3*1.5),2.0) << endl;
  t.test_rel(ti.interp(0.5,1.5),pow(sin(0.1*0.5+0.3*1.5),2.0),1.0e-1,
	     "2dintp x 1");
  cout << ti.interp(0.99,1.99) << " " 
       << pow(sin(0.1*0.99+0.3*1.99),2.0) << endl;
  t.test_rel(ti.interp(0.99,1.99),pow(sin(0.1*0.99+0.3*1.99),2.0),1.0e-2,
	     "2dintp x 2");
  cout << ti.interp(1.0,2.0) << " " 
       << pow(sin(0.1*1.0+0.3*2.0),2.0) << endl;
  t.test_rel(ti.interp(1.0,2.0),pow(sin(0.1*1.0+0.3*2.0),2.0),1.0e-8,
	     "2dintp x 3");
  cout << endl;

  ti.set_data(3,3,x,y,data,false);
  cout << ti.interp(0.5,1.5) << " " 
       << pow(sin(0.1*0.5+0.3*1.5),2.0) << endl;
  t.test_rel(ti.interp(0.5,1.5),pow(sin(0.1*0.5+0.3*1.5),2.0),1.0e-1,
	     "2dintp y 1");
  cout << ti.interp(0.99,1.99) << " " 
       << pow(sin(0.1*0.99+0.3*1.99),2.0) << endl;
  t.test_rel(ti.interp(0.99,1.99),pow(sin(0.1*0.99+0.3*1.99),2.0),1.0e-2,
	     "2dintp y 2");
  cout << ti.interp(1.0,2.0) << " " 
       << pow(sin(0.1*1.0+0.3*2.0),2.0) << endl;
  t.test_rel(ti.interp(1.0,2.0),pow(sin(0.1*1.0+0.3*2.0),2.0),1.0e-8,
	     "2dintp y 3");
  cout << endl;
  
  cout << ti.deriv_x(0.5,1.5) << " "
       << 2.0*sin(0.1*0.5+0.3*1.5)*0.1 << endl;
  cout << ti.deriv_y(0.5,1.5) << " "
       << 2.0*sin(0.1*0.5+0.3*1.5)*0.3 << endl;
  cout << ti.deriv2_x(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.1*0.1 << endl;
  cout << ti.deriv2_y(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.3*0.3 << endl;
  cout << ti.deriv_xy(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.1*0.3 << endl;
  cout << endl;

  const int size=21;
  ovector xn(size), yn(size);
  omatrix datan(size,size);
  for(i=0;i<size;i++) {
    xn[i]=((double)i)*(2.0/((double)(size-1)));
    yn[i]=3.0-((double)i)*(2.0/((double)(size-1)));
  }
  for(i=0;i<size;i++) {
    for(j=0;j<size;j++) {
      datan[i][j]=pow(sin(0.1*xn[j]+0.3*yn[i]),2.0);
    }
  }

  ti.set_data(size,size,xn,yn,datan,false);
  cout << ti.deriv_x(0.5,1.5) << " "
       << 2.0*sin(0.1*0.5+0.3*1.5)*0.1 << endl;
  cout << ti.deriv_y(0.5,1.5) << " "
       << 2.0*sin(0.1*0.5+0.3*1.5)*0.3 << endl;
  cout << ti.deriv2_x(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.1*0.1 << endl;
  cout << ti.deriv2_y(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.3*0.3 << endl;
  cout << ti.deriv_xy(0.5,1.5) << " "
       << 2.0*cos(0.1*0.5+0.3*1.5)*0.1*0.3 << endl;
  cout << endl;
  
  t.report();
  return 0;
}

