/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/sn_classical.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

//--------------------------------------------
// sn_classical class

sn_classical::sn_classical(double mass, double dof) : 
  part(mass,dof), deriv_classical(mass,dof) {
  
  name="sn_classical";
}

sn_classical::~sn_classical() {
}

int sn_classical::calc_mu(const double temper) {
  if (non_interacting==true) {nu=mu; ms=m;}
  if (nu/temper<-500.0) n=0.0;
  else n=exp(nu/temper)*g*pow(ms*temper/pi/2.0,1.5);
  ed=1.5*temper*n;
  pr=n*temper;
  en=(ed+pr-n*nu)/temper;
  dndT=-nu/temper/temper*n+1.5*n/temper;
  dndmu=n/temper;
  dsdT=2.5*dndT-nu*dndT/temper+n*nu/temper/temper;
  dndm=1.5*n/ms;
  return 0;
}

int sn_classical::calc_density(const double temper) {
  if (non_interacting==true) {ms=m;}
  if (n<=0.0) {
    nu=ms;
    if (non_interacting==true) {mu=nu;}
    ed=0.0;
    pr=0.0;
    en=0.0;
    if (n<0.0) return -1;
    else return 0;
  }
  nu=temper*log(n/g*pow(2.0*pi/ms/temper,1.5));
  if (non_interacting==true) {mu=nu;}
  ed=1.5*temper*n;
  pr=n*temper;
  en=(ed+pr-n*mu)/temper;
  dndT=-nu/temper/temper*n+1.5*n/temper;
  dndmu=n/temper;
  dsdT=2.5*dndT-nu*dndT/temper+n*nu/temper/temper;
  dndm=1.5*n/ms;
  return 0;
}

int sn_classical::pair_mu(const double temper) {
  return calc_mu(temper);
}

int sn_classical::pair_density(const double temper) {
  return calc_density(temper);
}

template<> int io_tlate<sn_classical>::input(cinput *co, in_file_format *ins, 
					     sn_classical *f) {
  deriv_classical *g=(deriv_classical *)f;
  std::string t;
  co->object_in("deriv_classical",ins,g,t);
  return 0;
}

template<> int io_tlate<sn_classical>::output(coutput *co, 
					      out_file_format *outs, 
					      sn_classical *f) {
  deriv_classical *g=(deriv_classical *)f;
  std::string t;
  co->object_out("deriv_classical",outs,g,0,0,t);
  return 0;
}

template<> const char *io_tlate<sn_classical>::type() {
  return "sn_classical";
}

