/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/sn_classical.h>
#include <o2scl/nonrel_fermion.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  cout.setf(ios::scientific);
  
  sn_classical sf(5.0,2.0);
  classical ef(5.0,2.0);

  t.test_rel(sf.m,5.0,1.0e-5,"mass_inheritance");
  
  double T=1.0;
  sf.non_interacting=false;
  sf.m=2.5;
  sf.nu=1.99*T-sf.m+sf.ms;
  ef.non_interacting=false;
  ef.m=2.5;
  ef.nu=1.99*T+sf.ms;
  
  cout << sf.ms << " " << ef.ms << endl;
  cout << sf.m << " " << ef.m << endl;

  cout << "\"Non-degenerate\": " << endl;
  sf.calc_mu(T);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  ef.calc_mu(T);
  cout << "ef: " << ef.n << " " << ef.ed-ef.n*ef.m << " " << ef.en << " " 
       << ef.pr << " " << ef.nu << endl;
  t.test_rel(sf.n,ef.n,1.0e-4,"ndeg density");
  t.test_rel(sf.ed,ef.ed-ef.n*ef.m,1.0e-4,"ndeg energy density");
  t.test_rel(sf.pr,ef.pr,1.0e-4,"ndeg pressure");
  t.test_rel(sf.en,ef.en,1.0e-4,"ndeg entropy");
  cout << endl;
  
  cout << "\"Degenerate\": " << endl;
  sf.nu=2.01*T-sf.m+sf.ms;
  ef.nu=2.01*T+sf.ms;
  sf.calc_mu(T);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << endl;
  ef.calc_mu(T);
  cout << "ef: " << ef.n << " " << ef.ed-ef.n*ef.m << " " << ef.en << " " 
       << ef.pr << endl;
  t.test_rel(sf.n,ef.n,1.0e-4,"deg density");
  t.test_rel(sf.ed,ef.ed-ef.n*ef.m,1.0e-4,"deg energy density");
  t.test_rel(sf.pr,ef.pr,1.0e-4,"deg pressure");
  t.test_rel(sf.en,ef.en,1.0e-4,"deg entropy");
  cout << endl;

  cout << "Test derivatives (\"non-degenerate\", direct): " << endl;
  double d1, d2, eps=1.0e-4;
  double dndmu, dndT, dsdT, dndm;
  
  sf.nu=1.0*T-sf.m+sf.ms+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.nu=1.0*T-sf.m+sf.ms;
  sf.calc_mu(T);
  d2=sf.n;
  dndmu=(d1-d2)/eps;

  sf.nu=1.0*T-sf.m+sf.ms;
  sf.calc_mu(T+eps);
  d1=sf.n;
  sf.calc_mu(T);
  d2=sf.n;
  dndT=(d1-d2)/eps;

  sf.calc_mu(T+eps);
  d1=sf.en;
  sf.calc_mu(T);
  d2=sf.en;
  dsdT=(d1-d2)/eps;

  sf.ms=5.0+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.ms=5.0;
  sf.calc_mu(T);
  d2=sf.n;
  dndm=(d1-d2)/eps;

  sf.calc_mu(T);
  cout << "sf: " << sf.dndmu << " " << sf.dndT << " " << sf.dsdT << " "
       << sf.dndm << endl;
  cout << "nu: " << dndmu << " " << dndT << " " << dsdT << " "
       << dndm << endl;
  t.test_rel(dndmu,sf.dndmu,5.0e-4,"ndeg dir dndmu");
  t.test_rel(dndT,sf.dndT,5.0e-4,"ndeg dir dndT");
  t.test_rel(dsdT,sf.dsdT,5.0e-4,"ndeg dir dsdT");
  t.test_rel(dndm,sf.dndm,5.0e-4,"ndeg dir dndm");
  cout << endl;

  cout << "Test derivatives (\"degenerate\", direct): " << endl;

  sf.nu=4.0*T-sf.m+sf.ms+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.nu=4.0*T-sf.m+sf.ms;
  sf.calc_mu(T);
  d2=sf.n;
  dndmu=(d1-d2)/eps;

  sf.nu=4.0*T-sf.m+sf.ms;
  sf.calc_mu(T+eps);
  d1=sf.n;
  sf.calc_mu(T);
  d2=sf.n;
  dndT=(d1-d2)/eps;

  sf.calc_mu(T+eps);
  d1=sf.en;
  sf.calc_mu(T);
  d2=sf.en;
  dsdT=(d1-d2)/eps;
  
  sf.ms=5.0+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.ms=5.0;
  sf.calc_mu(T);
  d2=sf.n;
  dndm=(d1-d2)/eps;

  sf.calc_mu(T);
  cout << "sf: " << sf.dndmu << " " << sf.dndT << " " << sf.dsdT << " "
       << sf.dndm << endl;
  cout << "nu: " << dndmu << " " << dndT << " " << dsdT << " "
       << dndm << endl;
  t.test_rel(dndmu,sf.dndmu,5.0e-4,"deg dir dndmu");
  t.test_rel(dndT,sf.dndT,5.0e-4,"deg dir dndT");
  t.test_rel(dsdT,sf.dsdT,5.0e-4,"deg dir dsdT");
  t.test_rel(dndm,sf.dndm,5.0e-4,"deg dir dndm");
  cout << endl;

  cout << "Test derivatives (\"non-degenerate\", byparts): " << endl;

  sf.nu=1.0*T-sf.m+sf.ms+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.nu=1.0*T-sf.m+sf.ms;
  sf.calc_mu(T);
  d2=sf.n;
  dndmu=(d1-d2)/eps;

  sf.nu=1.0*T-sf.m+sf.ms;
  sf.calc_mu(T+eps);
  d1=sf.n;
  sf.calc_mu(T);
  d2=sf.n;
  dndT=(d1-d2)/eps;

  sf.calc_mu(T+eps);
  d1=sf.en;
  sf.calc_mu(T);
  d2=sf.en;
  dsdT=(d1-d2)/eps;
  
  sf.ms=5.0+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.ms=5.0;
  sf.calc_mu(T);
  d2=sf.n;
  dndm=(d1-d2)/eps;

  sf.calc_mu(T);
  cout << "sf: " << sf.dndmu << " " << sf.dndT << " " << sf.dsdT << " "
       << sf.dndm << endl;
  cout << "nu: " << dndmu << " " << dndT << " " << dsdT << " "
       << dndm << endl;
  t.test_rel(dndmu,sf.dndmu,5.0e-4,"ndeg byp dndmu");
  t.test_rel(dndT,sf.dndT,5.0e-4,"ndeg byp dndT");
  t.test_rel(dsdT,sf.dsdT,5.0e-4,"ndeg byp dsdT");
  t.test_rel(dndm,sf.dndm,5.0e-4,"ndeg byp dndm");
  cout << endl;

  cout << "Test derivatives (\"degenerate\", byparts): " << endl;

  sf.nu=4.0*T-sf.m+sf.ms+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.nu=4.0*T-sf.m+sf.ms;
  sf.calc_mu(T);
  d2=sf.n;
  dndmu=(d1-d2)/eps;

  sf.nu=4.0*T-sf.m+sf.ms;
  sf.calc_mu(T+eps);
  d1=sf.n;
  sf.calc_mu(T);
  d2=sf.n;
  dndT=(d1-d2)/eps;

  sf.calc_mu(T+eps);
  d1=sf.en;
  sf.calc_mu(T);
  d2=sf.en;
  dsdT=(d1-d2)/eps;
  
  sf.ms=5.0+eps;
  sf.calc_mu(T);
  d1=sf.n;
  sf.ms=5.0;
  sf.calc_mu(T);
  d2=sf.n;
  dndm=(d1-d2)/eps;

  sf.calc_mu(T);
  cout << "sf: " << sf.dndmu << " " << sf.dndT << " " << sf.dsdT << " "
       << sf.dndm << endl;
  cout << "nu: " << dndmu << " " << dndT << " " << dsdT << " "
       << dndm << endl;
  t.test_rel(dndmu,sf.dndmu,5.0e-4,"deg byp dndmu");
  t.test_rel(dndT,sf.dndT,5.0e-4,"deg byp dndT");
  t.test_rel(dsdT,sf.dsdT,5.0e-4,"deg byp dsdT");
  t.test_rel(dndm,sf.dndm,5.0e-4,"deg byp dndm");
  cout << endl;

  cout << "Check calc_density()." << endl;
  sf.calc_mu(1.5);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.calc_density(1.5);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.calc_mu(0.7);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.calc_density(0.7);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;

  cout << "Check pair_density()." << endl;
  sf.pair_mu(1.5);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.pair_density(1.5);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.pair_mu(0.7);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  sf.pair_density(0.7);
  cout << "sf: " << sf.n << " " << sf.ed << " " << sf.en << " " 
       << sf.pr << " " << sf.nu << endl;
  cout << endl;

  t.report();
  return 0;
}
