/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GAUSSIAN_2D_H
#define O2SCL_GAUSSIAN_2D_H

#include <o2scl/rnga.h>

#ifndef DOXYGENP
namespace o2scl {
#endif
  
  /** 
      \brief Generate two random numbers from a normal distribution
      
      \todo Double check that sigma is implemented correctly
      
  */
  template<class rng_t> class gaussian_2d {
  public:
    
    rng_t r;

    /** \brief Generate two numbers from a 
	distribution with zero mean and standard deviation sigma
    */
    void random(double sigma, double &x, double &y) {
      
      double x1, x2, w, y1, y2;
      
      do {
	x1 = 2.0 * r.random() - 1.0;
	x2 = 2.0 * r.random() - 1.0;
	w = x1 * x1 + x2 * x2;
      } while ( w >= 1.0 );
      
      w = sqrt( (-2.0 * log( w ) ) / w );
      x = x1 * sigma * w;
      y = x2 * sigma * w;
      
      return;
    }
    
  };

#ifndef DOXYGENP
}
#endif

#endif
