/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/root.h>
#include <o2scl/cern_root.h>
#include <o2scl/cern_mroot_root.h>
#include <o2scl/gsl_root_stef.h>
#include <o2scl/gsl_root_brent.h>
#include <o2scl/mroot.h>
#include <o2scl/cern_mroot.h>
#include <o2scl/gsl_mroot_hybrids.h>

using namespace std;
using namespace o2scl;

template<> int io_tlate<root<void *,funct<void *> > >::input
(cinput *ci, in_file_format *ins, root<void *, funct<void *> > *ro) {
  ins->int_in(ro->verbose,"verbose");
  ins->int_in(ro->ntrial,"ntrial");
  ins->double_in(ro->tolf,"tolf");
  ins->double_in(ro->tolx,"tolx");
  return 0;
}

template<> int io_tlate<root<void *,funct<void *> > >::output
(coutput *co, out_file_format *outs, root<void *, funct<void *> > *ro) {
  outs->int_out(ro->verbose,"verbose");
  outs->int_out(ro->ntrial,"ntrial");
  outs->double_out(ro->tolf,"tolf");
  outs->double_out(ro->tolx,"tolx");
  return 0;
}

template<> const char *io_tlate<root<void *,funct<void *> > >::type() {
  return "root";
}

template<> int io_tlate<cern_root<void *,funct<void *> > >::input
(cinput *ci, in_file_format *ins, cern_root<void *, funct<void *> > *ro) {
  std::string t;
  ci->object_in("root",ins,ro,t);
  ins->int_in(ro->mode,"mode");
  return 0;
}

template<> int io_tlate<cern_root<void *,funct<void *> > >::output
(coutput *co, out_file_format *outs, cern_root<void *, funct<void *> > *ro) {
  std::string t;
  co->object_out("root",outs,ro,0,0,t);
  outs->int_out(ro->mode,"mode");
  return 0;
}

template<> const char *io_tlate<cern_root<void *,funct<void *> > >::type() {
  return "cern_root";
}

template<> int io_tlate<cern_mroot_root<void *,funct<void *> > >::input
(cinput *ci, in_file_format *ins, 
 cern_mroot_root<void *, funct<void *> > *ro) {
  std::string t;
  ci->object_in("root",ins,ro,t);
  ins->int_in(ro->maxf,"maxf");
  ins->double_in(ro->eps,"eps");
  ins->double_in(ro->scale,"scale");
  return 0;
}

template<> int io_tlate<cern_mroot_root<void *,funct<void *> > >::output
(coutput *co, out_file_format *outs, 
 cern_mroot_root<void *, funct<void *> > *ro) {
  std::string t;
  co->object_out("root",outs,ro,0,0,t);
  outs->int_out(ro->maxf,"maxf");
  outs->double_out(ro->eps,"eps");
  outs->double_out(ro->scale,"scale");
  return 0;
}

template<> const char *io_tlate<cern_mroot_root<void *,
						funct<void *> > >::type() {
  return "cern_mroot_root";
}

template<> int io_tlate<gsl_root_brent<void *,funct<void *> > >::input
(cinput *ci, in_file_format *ins, gsl_root_brent<void *, funct<void *> > *ro) {
  std::string t;
  ci->object_in("root",ins,ro,t);
  return 0;
}

template<> int io_tlate<gsl_root_brent<void *,funct<void *> > >::output
(coutput *co, out_file_format *outs, 
 gsl_root_brent<void *, funct<void *> > *ro) {
  std::string t;
  co->object_out("root",outs,ro,0,0,t);
  return 0;
}

template<> const char *io_tlate<gsl_root_brent<void *,
					       funct<void *> > >::type() {
  return "gsl_root_brent";
}

template<> int io_tlate<gsl_root_stef<void *,funct<void *>, 
				      funct<void *> > >::input
(cinput *ci, in_file_format *ins, 
 gsl_root_stef<void *, funct<void *>, funct<void *> > *ro) {
  std::string t;
  ci->object_in("root",ins,ro,t);
  return 0;
}

template<> int io_tlate<gsl_root_stef<void *,funct<void *>,
				      funct<void *> > >::output
(coutput *co, out_file_format *outs, 
 gsl_root_stef<void *, funct<void *>, funct<void *> > *ro) {
  std::string t;
  co->object_out("root",outs,ro,0,0,t);
  return 0;
}

template<> const char *io_tlate<gsl_root_stef<void *,
					      funct<void *>,
					      funct<void *> > >::type() {
  return "gsl_root_stef";
}

template<> int io_tlate<mroot<void *,mm_funct<void *> > >::input
(cinput *ci, in_file_format *ins, mroot<void *, mm_funct<void *> > *ro) {
  ins->int_in(ro->verbose,"verbose");
  ins->int_in(ro->ntrial,"ntrial");
  ins->double_in(ro->tolf,"tolf");
  ins->double_in(ro->tolx,"tolx");
  return 0;
}

template<> int io_tlate<mroot<void *,mm_funct<void *> > >::output
(coutput *co, out_file_format *outs, mroot<void *, mm_funct<void *> > *ro) {
  outs->int_out(ro->verbose,"verbose");
  outs->int_out(ro->ntrial,"ntrial");
  outs->double_out(ro->tolf,"tolf");
  outs->double_out(ro->tolx,"tolx");
  return 0;
}

template<> const char *io_tlate<mroot<void *,mm_funct<void *> > >::type() {
  return "mroot";
}

