(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37399,       1129]*)
(*NotebookOutlinePosition[     38390,       1159]*)
(*  CellTagsIndexPosition[     38346,       1155]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Expansions based on Johns, Ellis and Lattimer", "Section"],

Cell[BoxData[
    \(\(\(Off[General::"\<spell\>"]\)\(;\)\(\ \)\(Off[
      General::"\<spell1\>"]\)\(;\)\(\ \)\)\)], "Input"],

Cell[CellGroupData[{

Cell["Non-degenerate and non-relativstic:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pcoeff = \(t\^2\) \(\((\(-1\))\)\^\(n + 1\)\) 
        Exp[n \((\[Psi] + 1/t)\)]/n\^2\)], "Input"],

Cell[BoxData[
    \(\(\((\(-1\))\)\^\(1 + n\)\ \[ExponentialE]\^\(n\ \((1\/t + \[Psi])\)\)\ \
t\^2\)\/n\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(besselseries = 
      Simplify[Normal[Series[BesselK[2, x], {x, \[Infinity], 6}]]]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(-x\)\ \@\(\[Pi]\/2\)\ \((1\/x)\)\^\(13/2\)\ \
\((4729725 - 2162160\ x + 1330560\ x\^2 - 1290240\ x\^3 + 3440640\ x\^4 + \
7864320\ x\^5 + 4194304\ x\^6)\)\)\/4194304\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pNDNR = 
      Simplify[pcoeff\ \((besselseries /. x \[Rule] n/t)\)]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \@\(\[Pi]\/2\)\ \
\((t\/n)\)\^\(5/2\)\ \((4194304\ n\^6 + 7864320\ n\^5\ t + 
                3440640\ n\^4\ t\^2 - 1290240\ n\^3\ t\^3 + 
                1330560\ n\^2\ t\^4 - 2162160\ n\ t\^5 + 
                4729725\ t\^6)\)\)\/\(4194304\ n\^6\)\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
Separate out the first term to make the expansion more clear:\
\>", \
"Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(firstterm = \((Simplify[pNDNR/t\^\(5/2\), t > 0] /. t \[Rule] 0)\) 
        t\^\(5/2\)\)], "Input"],

Cell[BoxData[
    \(\(-\((\(-1\))\)\^n\)\ \[ExponentialE]\^\(n\ \[Psi]\)\ \((1\/n)\)\^\(5/2\
\)\ \@\(\[Pi]\/2\)\ t\^\(5/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pNDNRsimp = 
      Expand[Simplify[{firstterm, pNDNR/firstterm}, n > 0]]\)], "Input"],

Cell[BoxData[
    \({\(-\(\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \@\(\[Pi]\/2\)\
\ t\^2\ \@\(t\/n\)\)\/n\^2\)\), 
      1 + \(15\ t\)\/\(8\ n\) + \(105\ t\^2\)\/\(128\ n\^2\) - \(315\ \
t\^3\)\/\(1024\ n\^3\) + \(10395\ t\^4\)\/\(32768\ n\^4\) - \(135135\ \
t\^5\)\/\(262144\ n\^5\) + \(4729725\ t\^6\)\/\(4194304\ n\^6\)}\)], "Output"]
}, Open  ]],

Cell["Calculate the number and energy densities:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]NDNR = D[pNDNR, \[Psi]]/t\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \@\(\[Pi]\/2\)\ \
\((t\/n)\)\^\(5/2\)\ \((4194304\ n\^6 + 7864320\ n\^5\ t + 
                3440640\ n\^4\ t\^2 - 1290240\ n\^3\ t\^3 + 
                1330560\ n\^2\ t\^4 - 2162160\ n\ t\^5 + 
                4729725\ t\^6)\)\)\/\(4194304\ n\^5\ t\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ft2 = \((Simplify[\[Rho]NDNR/t\^\(3/2\), t > 0] /. t \[Rule] 0)\) 
        t\^\(3/2\); \[Rho]NDNRsimp = 
      Expand[Simplify[{ft2, \[Rho]NDNR/ft2}, t > 0]]\)], "Input"],

Cell[BoxData[
    \({\(-\(\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \@\(\[Pi]\/2\)\
\ t\ \@\(t\/n\)\)\/n\)\), 
      1 + \(15\ t\)\/\(8\ n\) + \(105\ t\^2\)\/\(128\ n\^2\) - \(315\ \
t\^3\)\/\(1024\ n\^3\) + \(10395\ t\^4\)\/\(32768\ n\^4\) - \(135135\ \
t\^5\)\/\(262144\ n\^5\) + \(4729725\ t\^6\)\/\(4194304\ n\^6\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]NDNR = 
      Simplify[t\ D[pNDNR, t] - pNDNR + \[Rho]NDNR]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(8388608\ n\^7\)\)\((\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \
\[Psi]\)\ \@\(\[Pi]\/2\)\ \((t\/n)\)\^\(3/2\)\ \((8388608\ n\^7 + 
              28311552\ n\^6\ t + 46202880\ n\^5\ t\^2 + 
              21504000\ n\^4\ t\^3 - 8951040\ n\^3\ t\^4 + 
              10311840\ n\^2\ t\^5 - 18648630\ n\ t\^6 + 
              70945875\ t\^7)\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ft3 = \((Simplify[\[CurlyEpsilon]NDNR/t\^\(3/2\), t > 0] /. 
            t \[Rule] 0)\) t\^\(3/2\); \[CurlyEpsilon]NDNRsimp = 
      Expand[Simplify[{ft3, \[CurlyEpsilon]NDNR/ft3}, t > 0]]\)], "Input"],

Cell[BoxData[
    \({\(-\(\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \@\(\[Pi]\/2\)\
\ t\ \@\(t\/n\)\)\/n\)\), 
      1 + \(27\ t\)\/\(8\ n\) + \(705\ t\^2\)\/\(128\ n\^2\) + \(2625\ t\^3\)\
\/\(1024\ n\^3\) - \(34965\ t\^4\)\/\(32768\ n\^4\) + \(322245\ \
t\^5\)\/\(262144\ n\^5\) - \(9324315\ t\^6\)\/\(4194304\ n\^6\) + \(70945875\ \
t\^7\)\/\(8388608\ n\^7\)}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Non-degenerate and extremely relativistic:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pcoeff = \(t\^2\) \(\((\(-1\))\)\^\(n + 1\)\) 
        Exp[n\ \[Psi]]/n\^2\)], "Input"],

Cell[BoxData[
    \(\(\((\(-1\))\)\^\(1 + n\)\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\^2\)\/n\^2\
\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(bs = Series[BesselK[2, x] Exp[x], {x, 0, 3}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(2\/x\^2\), "+", \(2\/x\), "+", \(1\/2\), "-", \(x\/6\), 
        "+", \(\((\(-\(7\/96\)\) - EulerGamma\/8 + Log[2]\/8 - 
              Log[x]\/8)\)\ x\^2\), 
        "+", \(\((13\/480 - EulerGamma\/8 + Log[2]\/8 - Log[x]\/8)\)\ x\^3\), 
        "+", 
        InterpretationBox[\(O[x]\^4\),
          SeriesData[ x, 0, {}, -2, 4, 1],
          Editable->False]}],
      SeriesData[ x, 0, {2, 2, 
        Rational[ 1, 2], 
        Rational[ -1, 6], 
        Plus[ 
          Rational[ -7, 96], 
          Times[ 
            Rational[ -1, 8], EulerGamma], 
          Times[ 
            Rational[ 1, 8], 
            Log[ 2]], 
          Times[ 
            Rational[ -1, 8], 
            Log[ x]]], 
        Plus[ 
          Rational[ 13, 480], 
          Times[ 
            Rational[ -1, 8], EulerGamma], 
          Times[ 
            Rational[ 1, 8], 
            Log[ 2]], 
          Times[ 
            Rational[ -1, 8], 
            Log[ x]]]}, -2, 4, 1],
      Editable->False]], "Output"]
}, Open  ]],

Cell["Separate the individual terms:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bs2 = {Normal[Series[BesselK[2, x] Exp[x], {x, 0, 1}]], 
        SeriesCoefficient[bs, 2], SeriesCoefficient[bs, 3]}\)], "Input"],

Cell[BoxData[
    \({1\/2 + 2\/x\^2 + 2\/x - x\/6, \(-\(7\/96\)\) - EulerGamma\/8 + 
        Log[2]\/8 - Log[x]\/8, 
      13\/480 - EulerGamma\/8 + Log[2]\/8 - Log[x]\/8}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pNDER = Simplify[pcoeff\ \((bs2 /. x \[Rule] n/t)\)]\)], "Input"],

Cell[BoxData[
    \({\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\ \((n\^3 - 3\ \
n\^2\ t - 12\ n\ t\^2 - 12\ t\^3)\)\)\/\(6\ n\^4\), \(\((\(-1\))\)\^n\ \
\[ExponentialE]\^\(n\ \[Psi]\)\ t\^2\ \((7 + 12\ EulerGamma - 12\ Log[2] + 12\
\ Log[n\/t])\)\)\/\(96\ n\^2\), \(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \
\[Psi]\)\ t\^2\ \((\(-13\) + 60\ EulerGamma - 60\ Log[2] + 60\ Log[n\/t])\)\)\
\/\(480\ n\^2\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefx = \(-2\)\ \(t\^4\) 
        Exp[n\ \[Psi]]\ \((\(-1\))\)\^n/n\^4\)], "Input"],

Cell[BoxData[
    \(\(-\(\(2\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\^4\)\/n\^4\
\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pNDERsimp = Expand[{coefx, pNDER/coefx}]\)], "Input"],

Cell[BoxData[
    \({\(-\(\(2\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \
t\^4\)\/n\^4\)\), {1 - n\^3\/\(12\ t\^3\) + n\^2\/\(4\ t\^2\) + 
          n\/t, \(-\(\(7\ n\^2\)\/\(192\ t\^2\)\)\) - \(EulerGamma\ \
n\^2\)\/\(16\ t\^2\) + \(n\^2\ Log[2]\)\/\(16\ t\^2\) - \(n\^2\ Log[n\/t]\)\/\
\(16\ t\^2\), \(13\ n\^2\)\/\(960\ t\^2\) - \(EulerGamma\ n\^2\)\/\(16\ \
t\^2\) + \(n\^2\ Log[2]\)\/\(16\ t\^2\) - \(n\^2\ Log[n\/t]\)\/\(16\ t\^2\)}}\
\)], "Output"]
}, Open  ]],

Cell["Compute the number and energy densities:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]NDER = D[pNDER, \[Psi]]/t\)], "Input"],

Cell[BoxData[
    \({\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \((n\^3 - 3\ n\^2\ \
t - 12\ n\ t\^2 - 12\ t\^3)\)\)\/\(6\ n\^3\), \(\((\(-1\))\)\^n\ \
\[ExponentialE]\^\(n\ \[Psi]\)\ t\ \((7 + 12\ EulerGamma - 12\ Log[2] + 12\ \
Log[n\/t])\)\)\/\(96\ n\), \(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \
t\ \((\(-13\) + 60\ EulerGamma - 60\ Log[2] + 60\ Log[n\/t])\)\)\/\(480\ n\)}\
\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefx2 = \(-2\)\ \(t\^3\) 
        Exp[n\ \[Psi]]\ \((\(-1\))\)\^n/n\^3\)], "Input"],

Cell[BoxData[
    \(\(-\(\(2\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\^3\)\/n\^3\
\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]NDERsimp = {coefx2, Expand[\[Rho]NDER/coefx2]}\)], "Input"],

Cell[BoxData[
    \({\(-\(\(2\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \
t\^3\)\/n\^3\)\), {1 - n\^3\/\(12\ t\^3\) + n\^2\/\(4\ t\^2\) + 
          n\/t, \(-\(\(7\ n\^2\)\/\(192\ t\^2\)\)\) - \(EulerGamma\ \
n\^2\)\/\(16\ t\^2\) + \(n\^2\ Log[2]\)\/\(16\ t\^2\) - \(n\^2\ Log[n\/t]\)\/\
\(16\ t\^2\), \(13\ n\^2\)\/\(960\ t\^2\) - \(EulerGamma\ n\^2\)\/\(16\ \
t\^2\) + \(n\^2\ Log[2]\)\/\(16\ t\^2\) - \(n\^2\ Log[n\/t]\)\/\(16\ t\^2\)}}\
\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]NDER = 
      Simplify[t\ D[pNDER, t] - pNDER + \[Rho]NDER]\)], "Input"],

Cell[BoxData[
    \({\(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \((n\^4 - 3\ n\^3\ \
t - 15\ n\^2\ t\^2 - 36\ n\ t\^3 - 36\ t\^4)\)\)\/\(6\ n\^4\), \(\((\(-1\))\)\
\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\ \((t\ \((\(-5\) + 12\ EulerGamma - 12\
\ Log[2])\) + n\ \((7 + 12\ EulerGamma - 12\ Log[2])\) + 12\ \((n + t)\)\ \
Log[n\/t])\)\)\/\(96\ n\^2\), \(\((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\
\)\ t\ \((t\ \((\(-73\) + 60\ EulerGamma - 60\ Log[2])\) + n\ \((\(-13\) + 60\
\ EulerGamma - 60\ Log[2])\) + 60\ \((n + t)\)\ Log[n\/t])\)\)\/\(480\ \
n\^2\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefx3 = \(-6\)\ \(t\^3\) 
        Exp[n\ \[Psi]] \((\(-1\))\)\^n/n\^3\)], "Input"],

Cell[BoxData[
    \(\(-\(\(6\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ t\^3\)\/n\^3\
\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]NDERsimp = {coefx3, 
        Expand[\[CurlyEpsilon]NDER/coefx3]}\)], "Input"],

Cell[BoxData[
    \({\(-\(\(6\ \((\(-1\))\)\^n\ \[ExponentialE]\^\(n\ \[Psi]\)\ \
t\^3\)\/n\^3\)\), {1 - n\^3\/\(36\ t\^3\) + 
          n\^2\/\(12\ t\^2\) + \(5\ n\)\/\(12\ t\) + 
          t\/n, \(-\(\(7\ n\^2\)\/\(576\ t\^2\)\)\) - \(EulerGamma\ \
n\^2\)\/\(48\ t\^2\) + \(5\ n\)\/\(576\ t\) - \(EulerGamma\ n\)\/\(48\ t\) + \
\(n\^2\ Log[2]\)\/\(48\ t\^2\) + \(n\ Log[2]\)\/\(48\ t\) - \(n\^2\ Log[n\/t]\
\)\/\(48\ t\^2\) - \(n\ Log[n\/t]\)\/\(48\ t\), \(13\ n\^2\)\/\(2880\ t\^2\) \
- \(EulerGamma\ n\^2\)\/\(48\ t\^2\) + \(73\ n\)\/\(2880\ t\) - \(EulerGamma\ \
n\)\/\(48\ t\) + \(n\^2\ Log[2]\)\/\(48\ t\^2\) + \(n\ Log[2]\)\/\(48\ t\) - \
\(n\^2\ Log[n\/t]\)\/\(48\ t\^2\) - \(n\ Log[n\/t]\)\/\(48\ t\)}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The extremely degenerate case:", "Subsubsection"],

Cell["The pressure from JEL:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = 
      1/3\ Integrate[
          f[l] \(l\^4/
                Sqrt[l\^2 + 1]\)/\((1 + 
                  Exp[\((Sqrt[l\^2 + 1] - 1)\)/t - \[Psi]])\), {l, 
            0, \[Infinity]}]; \ p /. f[l] \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(1\/3\ \(\[Integral]\_0\%\[Infinity]\( l\^4\/\(\((1 + \
\[ExponentialE]\^\(\(\(-1\) + \@\(1 + l\^2\)\)\/t - \[Psi]\))\)\ \@\(1 + l\^2\
\)\)\) \[DifferentialD]l\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Change variables to ",
  Cell[BoxData[
      \(TraditionalForm\`l = \@\(\((z + 1)\)\^2 - 1\)\)]],
  "and \[Psi] t=x, and re-examine the integrand:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(dldz = D[Sqrt[\((z + 1)\)\^2 - 1], z]\)], "Input"],

Cell[BoxData[
    \(\(1 + z\)\/\@\(\(-1\) + \((1 + z)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(f[l] dl \[Equal] f[z] \((dl/dz)\)\ dz\)], "Input"],

Cell[BoxData[
    \(dl\ f[l] \[Equal] dl\ f[z]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(degiand = 
      Simplify[\(\(l\^4\) \(\(dldz/3\)/
                  Sqrt[l\^2 + 1]\)/\((1 + 
                    Exp[\((Sqrt[l\^2 + 1] - 1)\)/t - \[Psi]])\) /. 
            l \[Rule] Sqrt[\((z + 1)\)\^2 - 1]\) /. \[Psi] \[Rule] x/t, {z > 
            0}]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(x\/t\)\ \((z\ \((2 + z)\))\)\^\(3/2\)\)\/\(3\ \((\
\[ExponentialE]\^\(x\/t\) + \[ExponentialE]\^\(z\/t\))\)\)\)], "Output"]
}, Open  ]],

Cell["Note that the limits are the same.", "Text"],

Cell["Separate out the Fermi function:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(newf = 
      degiand/Exp[
            x/t]\ \((\[ExponentialE]\^\(x\/t\) + \[ExponentialE]\^\(z\/t\))\)\
\)], "Input"],

Cell[BoxData[
    \(1\/3\ \((z\ \((2 + z)\))\)\^\(3/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[degiand - newf/\((1 + Exp[z/t - x/t])\)]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
Now we can use the Sommerfeld expansion from Landau and Lifshitz, \
which is valid when \[Mu]/T (=x/t) is large.
Note that this is an asymptotic expansion, so we have to be careful about \
including too many terms.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(eq = 
      Integrate[
          f[\[Epsilon]]/\((1 + 
                Exp[\((\[Epsilon] - \[Mu])\)/T])\), {\[Epsilon], 
            0, \[Infinity]}] == 
        Expand[Integrate[f[\[Epsilon]], {\[Epsilon], 0, \[Mu]}] + 
            Integrate[
              T\ Normal[
                    Series[\((f[\[Mu] + T\ z] - f[\[Mu] - T\ z])\), {z, 0, 
                        5}]]/\((Exp[z] + 1)\), {z, 
                0, \[Infinity]}]]\)], "Input"],

Cell[BoxData[
    RowBox[{\(\[Integral]\_0\%\[Infinity]\( 
                f[\[Epsilon]]\/\(1 + \[ExponentialE]\^\(\(\[Epsilon] - \
\[Mu]\)\/T\)\)\) \[DifferentialD]\[Epsilon]\), "\[Equal]", 
      RowBox[{\(\[Integral]\_0\%\[Mu] 
              f[\[Epsilon]] \[DifferentialD]\[Epsilon]\), "+", 
        RowBox[{\(1\/6\), " ", \(\[Pi]\^2\), " ", \(T\^2\), " ", 
          RowBox[{
            SuperscriptBox["f", "\[Prime]",
              MultilineFunction->None], "[", "\[Mu]", "]"}]}], "+", 
        RowBox[{\(7\/360\), " ", \(\[Pi]\^4\), " ", \(T\^4\), " ", 
          RowBox[{
            SuperscriptBox["f", 
              TagBox[\((3)\),
                Derivative],
              MultilineFunction->None], "[", "\[Mu]", "]"}]}], "+", 
        FractionBox[
          RowBox[{"31", " ", \(\[Pi]\^6\), " ", \(T\^6\), " ", 
            RowBox[{
              SuperscriptBox["f", 
                TagBox[\((5)\),
                  Derivative],
                MultilineFunction->None], "[", "\[Mu]", "]"}]}], 
          "15120"]}]}]], "Output"]
}, Open  ]],

Cell["\<\
We add coefficients \[Alpha]1 and \[Alpha]2 to see how the \
Sommerfeld expansion contributes
(we'll set them to one later)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ped = 
      Simplify[Integrate[newf, {z, 0, x}] + \[Alpha]1\ \(\[Pi]\^2\) 
              t\^2/6 \((D[newf, z] /. z \[Rule] x)\) + 
            7\ \[Alpha]2\ \[Pi]\^4\ t\^4/
                360 \((D[D[D[newf, z], z], z] /. z \[Rule] x)\), x > 0] /. 
        x \[Rule] \[Psi]\ t\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(360\ \((t\ \[Psi]\ \((2 + t\ \[Psi])\))\)\^\(3/2\)\)\)\((\((1 \
+ t\ \[Psi])\)\ \((\(-7\)\ \[Pi]\^4\ t\^4\ \[Alpha]2 + 
              28\ \[Pi]\^4\ t\^5\ \[Alpha]2\ \[Psi] + 
              2\ t\^2\ \((\(-90\) + 120\ \[Pi]\^2\ t\^2\ \[Alpha]1 + 
                    7\ \[Pi]\^4\ t\^4\ \[Alpha]2)\)\ \[Psi]\^2 + 
              60\ t\^3\ \((1 + 4\ \[Pi]\^2\ t\^2\ \[Alpha]1)\)\ \[Psi]\^3 + 
              15\ t\^4\ \((21 + 4\ \[Pi]\^2\ t\^2\ \[Alpha]1)\)\ \[Psi]\^4 + 
              180\ t\^5\ \[Psi]\^5 + 30\ t\^6\ \[Psi]\^6)\) + 
        90\ \((2\ \@\(t\^3\ \[Psi]\^3\ \((2 + t\ \[Psi])\)\) + \@\(t\^5\ \
\[Psi]\^5\ \((2 + t\ \[Psi])\)\))\)\ ArcSinh[\@\(t\ \[Psi]\)\/\@2])\)\)\)], \
"Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The extremely degenerate and non-relativistic case:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pEDNR = 
      Expand[Simplify[
          Normal[Series[ped, {t, 0, 5}]], {\[Psi] > 0}]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(7\ \[Pi]\^4\ t\^\(5/2\)\ \[Alpha]2\)\/\(720\ \@2\ \
\[Psi]\^\(3/2\)\)\)\) + \(7\ \[Pi]\^4\ t\^\(7/2\)\ \[Alpha]2\)\/\(192\ \@2\ \
\@\[Psi]\) + \(\[Pi]\^2\ t\^\(5/2\)\ \[Alpha]1\ \@\[Psi]\)\/\(3\ \@2\) + \(49\
\ \[Pi]\^4\ t\^\(9/2\)\ \[Alpha]2\ \@\[Psi]\)\/\(1536\ \@2\) + \(5\ \[Pi]\^2\ \
t\^\(7/2\)\ \[Alpha]1\ \[Psi]\^\(3/2\)\)\/\(12\ \@2\) + 
      4\/15\ \@2\ t\^\(5/2\)\ \[Psi]\^\(5/2\) + \(7\ \[Pi]\^2\ t\^\(9/2\)\ \
\[Alpha]1\ \[Psi]\^\(5/2\)\)\/\(96\ \@2\) + 
      1\/7\ \@2\ t\^\(7/2\)\ \[Psi]\^\(7/2\) + \(t\^\(9/2\)\ \
\[Psi]\^\(9/2\)\)\/\(36\ \@2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefy = 4\ Sqrt[2]\ \(\[Psi]\^\(5/2\)\) t\^\(5/2\)/15\)], "Input"],

Cell[BoxData[
    \(4\/15\ \@2\ t\^\(5/2\)\ \[Psi]\^\(5/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pEDNRsimp = {coefy, Expand[pEDNR/coefy]}\)], "Input"],

Cell[BoxData[
    \({4\/15\ \@2\ t\^\(5/2\)\ \[Psi]\^\(5/2\), 
      1 + 35\/256\ \[Pi]\^2\ t\^2\ \[Alpha]1 - \(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(384\ \[Psi]\^4\) + \(35\ \[Pi]\^4\ t\ \[Alpha]2\)\/\(512\ \
\[Psi]\^3\) + \(5\ \[Pi]\^2\ \[Alpha]1\)\/\(8\ \[Psi]\^2\) + \(245\ \[Pi]\^4\ \
t\^2\ \[Alpha]2\)\/\(4096\ \[Psi]\^2\) + \(25\ \[Pi]\^2\ t\ \[Alpha]1\)\/\(32\
\ \[Psi]\) + \(15\ t\ \[Psi]\)\/28 + \(5\ t\^2\ \[Psi]\^2\)\/96}\)], "Output"]
}, Open  ]],

Cell["There appears to be a typo in the reference?", "Text"],

Cell["Compute the number and energy densities:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]EDNR = Expand[D[pEDNR, \[Psi]]/t]\)], "Input"],

Cell[BoxData[
    \(\(7\ \[Pi]\^4\ t\^\(3/2\)\ \[Alpha]2\)\/\(480\ \@2\ \[Psi]\^\(5/2\)\) - \
\(7\ \[Pi]\^4\ t\^\(5/2\)\ \[Alpha]2\)\/\(384\ \@2\ \[Psi]\^\(3/2\)\) + \(\
\[Pi]\^2\ t\^\(3/2\)\ \[Alpha]1\)\/\(6\ \@2\ \@\[Psi]\) + \(49\ \[Pi]\^4\ t\^\
\(7/2\)\ \[Alpha]2\)\/\(3072\ \@2\ \@\[Psi]\) + \(5\ \[Pi]\^2\ t\^\(5/2\)\ \
\[Alpha]1\ \@\[Psi]\)\/\(8\ \@2\) + 
      2\/3\ \@2\ t\^\(3/2\)\ \[Psi]\^\(3/2\) + \(35\ \[Pi]\^2\ t\^\(7/2\)\ \
\[Alpha]1\ \[Psi]\^\(3/2\)\)\/\(192\ \@2\) + \(t\^\(5/2\)\ \
\[Psi]\^\(5/2\)\)\/\@2 + \(t\^\(7/2\)\ \[Psi]\^\(7/2\)\)\/\(8\ \@2\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefy2 = 2\ Sqrt[2]/3\ \(t\^\(3/2\)\) \[Psi]\^\(3/2\)\)], "Input"],

Cell[BoxData[
    \(2\/3\ \@2\ t\^\(3/2\)\ \[Psi]\^\(3/2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]EDNRsimp = {coefy2, Expand[\[Rho]EDNR/coefy2]}\)], "Input"],

Cell[BoxData[
    \({2\/3\ \@2\ t\^\(3/2\)\ \[Psi]\^\(3/2\), 
      1 + 35\/256\ \[Pi]\^2\ t\^2\ \[Alpha]1 + \(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(640\ \[Psi]\^4\) - \(7\ \[Pi]\^4\ t\ \[Alpha]2\)\/\(512\ \
\[Psi]\^3\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(8\ \[Psi]\^2\) + \(49\ \[Pi]\^4\ \
t\^2\ \[Alpha]2\)\/\(4096\ \[Psi]\^2\) + \(15\ \[Pi]\^2\ t\ \[Alpha]1\)\/\(32\
\ \[Psi]\) + \(3\ t\ \[Psi]\)\/4 + \(3\ t\^2\ \[Psi]\^2\)\/32}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]EDNR = 
      Expand[Simplify[t\ D[pEDNR, t] - pEDNR + \[Rho]EDNR]]\)], "Input"],

Cell[BoxData[
    \(\(7\ \[Pi]\^4\ t\^\(3/2\)\ \[Alpha]2\)\/\(480\ \@2\ \[Psi]\^\(5/2\)\) - \
\(21\ \[Pi]\^4\ t\^\(5/2\)\ \[Alpha]2\)\/\(640\ \@2\ \[Psi]\^\(3/2\)\) + \(\
\[Pi]\^2\ t\^\(3/2\)\ \[Alpha]1\)\/\(6\ \@2\ \@\[Psi]\) + \(329\ \[Pi]\^4\ \
t\^\(7/2\)\ \[Alpha]2\)\/\(3072\ \@2\ \@\[Psi]\) + \(9\ \[Pi]\^2\ t\^\(5/2\)\ \
\[Alpha]1\ \@\[Psi]\)\/\(8\ \@2\) + \(343\ \[Pi]\^4\ t\^\(9/2\)\ \[Alpha]2\ \
\@\[Psi]\)\/\(3072\ \@2\) + 
      2\/3\ \@2\ t\^\(3/2\)\ \[Psi]\^\(3/2\) + \(235\ \[Pi]\^2\ t\^\(7/2\)\ \
\[Alpha]1\ \[Psi]\^\(3/2\)\)\/\(192\ \@2\) + \(9\ t\^\(5/2\)\ \[Psi]\^\(5/2\)\
\)\/\(5\ \@2\) + \(49\ \[Pi]\^2\ t\^\(9/2\)\ \[Alpha]1\ \
\[Psi]\^\(5/2\)\)\/\(192\ \@2\) + \(47\ t\^\(7/2\)\ \[Psi]\^\(7/2\)\)\/\(56\ \
\@2\) + \(7\ t\^\(9/2\)\ \[Psi]\^\(9/2\)\)\/\(72\ \@2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]EDNRsimp = {coefy2, 
        Expand[\[CurlyEpsilon]EDNR/coefy2]}\)], "Input"],

Cell[BoxData[
    \({2\/3\ \@2\ t\^\(3/2\)\ \[Psi]\^\(3/2\), 
      1 + 235\/256\ \[Pi]\^2\ t\^2\ \[Alpha]1 + \(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(640\ \[Psi]\^4\) - \(63\ \[Pi]\^4\ t\ \[Alpha]2\)\/\(2560\ \
\[Psi]\^3\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(8\ \[Psi]\^2\) + \(329\ \[Pi]\^4\ \
t\^2\ \[Alpha]2\)\/\(4096\ \[Psi]\^2\) + \(27\ \[Pi]\^2\ t\ \[Alpha]1\)\/\(32\
\ \[Psi]\) + \(343\ \[Pi]\^4\ t\^3\ \[Alpha]2\)\/\(4096\ \[Psi]\) + \(27\ t\ \
\[Psi]\)\/20 + 
        49\/256\ \[Pi]\^2\ t\^3\ \[Alpha]1\ \[Psi] + \(141\ t\^2\ \[Psi]\^2\)\
\/224 + \(7\ t\^3\ \[Psi]\^3\)\/96}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The extremely degenerate and extremely relativistic case:", \
"Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p2 = 
      Expand[Simplify[
          Normal[Series[ped, {t, \[Infinity], \(-1\)}]], \[Psi] > 
            0]]\)], "Input"],

Cell[BoxData[
    \(1\/12\ \[Pi]\^2\ t\^2\ \[Alpha]1 + 
      7\/180\ \[Pi]\^4\ t\^4\ \[Alpha]2 - \(t\ \[Psi]\)\/6 + 
      1\/3\ \[Pi]\^2\ t\^3\ \[Alpha]1\ \[Psi] + \(t\^2\ \[Psi]\^2\)\/4 + 
      1\/6\ \[Pi]\^2\ t\^4\ \[Alpha]1\ \[Psi]\^2 + \(t\^3\ \[Psi]\^3\)\/3 + \
\(t\^4\ \[Psi]\^4\)\/12\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(p2big = Series[ped, {t, \[Infinity], 2}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pEDER = 
      Simplify[{Normal[Series[ped, {t, \[Infinity], \(-1\)}]], 
          SeriesCoefficient[p2big, 0], SeriesCoefficient[p2big, 1], 
          SeriesCoefficient[p2big, 2]}, \[Psi] > 0]\)], "Input"],

Cell[BoxData[
    \({1\/180\ t\ \((7\ \[Pi]\^4\ t\^3\ \[Alpha]2 + 
            15\ \[Pi]\^2\ t\ \[Alpha]1\ \((1 + 4\ t\ \[Psi] + 
                  2\ t\^2\ \[Psi]\^2)\) + 
            15\ \[Psi]\ \((\(-2\) + 3\ t\ \[Psi] + 4\ t\^2\ \[Psi]\^2 + 
                  t\^3\ \[Psi]\^3)\))\), 
      1\/480\ \((\(-35\) - \(7\ \[Pi]\^4\ \[Alpha]2\)\/\[Psi]\^4 - \(10\ \
\[Pi]\^2\ \[Alpha]1\)\/\[Psi]\^2 - 60\ Log[1\/t] + 
            60\ Log[2\ \[Psi]])\), \(7\ \[Pi]\^4\ \[Alpha]2 + 5\ \[Pi]\^2\ \
\[Alpha]1\ \[Psi]\^2 + 15\ \[Psi]\^4\)\/\(120\ \[Psi]\^5\), \(-\(\(7\ \((7\ \
\[Pi]\^4\ \[Alpha]2 + 3\ \[Pi]\^2\ \[Alpha]1\ \[Psi]\^2 + 
                  3\ \[Psi]\^4)\)\)\/\(288\ \[Psi]\^6\)\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefz = \(\[Psi]\^4\) t\^4/12\)], "Input"],

Cell[BoxData[
    \(\(t\^4\ \[Psi]\^4\)\/12\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pEDERsimp = {coefz, Expand[pEDER/coefz]}\)], "Input"],

Cell[BoxData[
    \({\(t\^4\ \[Psi]\^4\)\/12, {1 + \(\[Pi]\^2\ \[Alpha]1\)\/\(t\^2\ \
\[Psi]\^4\) + \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(15\ \[Psi]\^4\) - 
          2\/\(t\^3\ \[Psi]\^3\) + \(4\ \[Pi]\^2\ \[Alpha]1\)\/\(t\ \[Psi]\^3\
\) + 3\/\(t\^2\ \[Psi]\^2\) + \(2\ \[Pi]\^2\ \[Alpha]1\)\/\[Psi]\^2 + 
          4\/\(t\ \[Psi]\), \(-\(\(7\ \[Pi]\^4\ \[Alpha]2\)\/\(40\ t\^4\ \
\[Psi]\^8\)\)\) - \(\[Pi]\^2\ \[Alpha]1\)\/\(4\ t\^4\ \[Psi]\^6\) - 
          7\/\(8\ t\^4\ \[Psi]\^4\) - \(3\ Log[1\/t]\)\/\(2\ t\^4\ \
\[Psi]\^4\) + \(3\ Log[2\ \[Psi]]\)\/\(2\ t\^4\ \[Psi]\^4\), \(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(10\ t\^4\ \[Psi]\^9\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(2\ t\^4\ \
\[Psi]\^7\) + 
          3\/\(2\ t\^4\ \[Psi]\^5\), \(-\(\(49\ \[Pi]\^4\ \[Alpha]2\)\/\(24\ \
t\^4\ \[Psi]\^10\)\)\) - \(7\ \[Pi]\^2\ \[Alpha]1\)\/\(8\ t\^4\ \[Psi]\^8\) - 
          7\/\(8\ t\^4\ \[Psi]\^6\)}}\)], "Output"]
}, Open  ]],

Cell["Compute the number and energy densities:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]EDER = Expand[D[pEDER, \[Psi]]/t]\)], "Input"],

Cell[BoxData[
    \({\(-\(1\/6\)\) + 1\/3\ \[Pi]\^2\ t\^2\ \[Alpha]1 + \(t\ \[Psi]\)\/2 + 
        1\/3\ \[Pi]\^2\ t\^3\ \[Alpha]1\ \[Psi] + 
        t\^2\ \[Psi]\^2 + \(t\^3\ \[Psi]\^3\)\/3, \(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(120\ t\ \[Psi]\^5\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(24\ t\ \[Psi]\
\^3\) + 1\/\(8\ t\ \[Psi]\), \(-\(\(7\ \[Pi]\^4\ \[Alpha]2\)\/\(24\ t\ \[Psi]\
\^6\)\)\) - \(\[Pi]\^2\ \[Alpha]1\)\/\(8\ t\ \[Psi]\^4\) - 
        1\/\(8\ t\ \[Psi]\^2\), \(49\ \[Pi]\^4\ \[Alpha]2\)\/\(48\ t\ \
\[Psi]\^7\) + \(7\ \[Pi]\^2\ \[Alpha]1\)\/\(24\ t\ \[Psi]\^5\) + 
        7\/\(48\ t\ \[Psi]\^3\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefz2 = \(t\^3\) \[Psi]\^3/3\)], "Input"],

Cell[BoxData[
    \(\(t\^3\ \[Psi]\^3\)\/3\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Rho]EDERsimp = {coefz2, Expand[\[Rho]EDER/coefz2]}\)], "Input"],

Cell[BoxData[
    \({\(t\^3\ \[Psi]\^3\)\/3, {1 - 
          1\/\(2\ t\^3\ \[Psi]\^3\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(t\ \[Psi]\^3\
\) + 3\/\(2\ t\^2\ \[Psi]\^2\) + \(\[Pi]\^2\ \[Alpha]1\)\/\[Psi]\^2 + 
          3\/\(t\ \[Psi]\), \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(40\ t\^4\ \[Psi]\^8\
\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(8\ t\^4\ \[Psi]\^6\) + 
          3\/\(8\ t\^4\ \[Psi]\^4\), \(-\(\(7\ \[Pi]\^4\ \[Alpha]2\)\/\(8\ \
t\^4\ \[Psi]\^9\)\)\) - \(3\ \[Pi]\^2\ \[Alpha]1\)\/\(8\ t\^4\ \[Psi]\^7\) - 
          3\/\(8\ t\^4\ \[Psi]\^5\), \(49\ \[Pi]\^4\ \[Alpha]2\)\/\(16\ t\^4\ \
\[Psi]\^10\) + \(7\ \[Pi]\^2\ \[Alpha]1\)\/\(8\ t\^4\ \[Psi]\^8\) + 
          7\/\(16\ t\^4\ \[Psi]\^6\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]EDER = 
      Expand[Simplify[t\ D[pEDER, t] - pEDER + \[Rho]EDER]]\)], "Input"],

Cell[BoxData[
    \({\(-\(1\/6\)\) + 5\/12\ \[Pi]\^2\ t\^2\ \[Alpha]1 + 
        7\/60\ \[Pi]\^4\ t\^4\ \[Alpha]2 + \(t\ \[Psi]\)\/2 + \[Pi]\^2\ t\^3\ \
\[Alpha]1\ \[Psi] + \(5\ t\^2\ \[Psi]\^2\)\/4 + 
        1\/2\ \[Pi]\^2\ t\^4\ \[Alpha]1\ \[Psi]\^2 + 
        t\^3\ \[Psi]\^3 + \(t\^4\ \[Psi]\^4\)\/4, 
      19\/96 + \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(120\ t\ \[Psi]\^5\) + \(7\ \[Pi]\
\^4\ \[Alpha]2\)\/\(480\ \[Psi]\^4\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(24\ t\ \
\[Psi]\^3\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(48\ \[Psi]\^2\) + 
        1\/\(8\ t\ \[Psi]\) + 1\/8\ Log[1\/t] - 
        1\/8\ Log[
            2\ \[Psi]], \(-\(\(7\ \[Pi]\^4\ \[Alpha]2\)\/\(24\ t\ \[Psi]\^6\)\
\)\) - \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(120\ \[Psi]\^5\) - \(\[Pi]\^2\ \[Alpha]1\
\)\/\(8\ t\ \[Psi]\^4\) - \(\[Pi]\^2\ \[Alpha]1\)\/\(24\ \[Psi]\^3\) - 
        1\/\(8\ t\ \[Psi]\^2\) - 
        1\/\(8\ \[Psi]\), \(49\ \[Pi]\^4\ \[Alpha]2\)\/\(48\ t\ \[Psi]\^7\) + \
\(49\ \[Pi]\^4\ \[Alpha]2\)\/\(288\ \[Psi]\^6\) + \(7\ \[Pi]\^2\ \
\[Alpha]1\)\/\(24\ t\ \[Psi]\^5\) + \(7\ \[Pi]\^2\ \[Alpha]1\)\/\(96\ \
\[Psi]\^4\) + 7\/\(48\ t\ \[Psi]\^3\) + 7\/\(96\ \[Psi]\^2\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(coefz3 = \(\[Psi]\^4\) t\^4/4\)], "Input"],

Cell[BoxData[
    \(\(t\^4\ \[Psi]\^4\)\/4\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({coefz3, Expand[\[CurlyEpsilon]EDER/coefz3]}\)], "Input"],

Cell[BoxData[
    \({\(t\^4\ \[Psi]\^4\)\/4, {1 - 
          2\/\(3\ t\^4\ \[Psi]\^4\) + \(5\ \[Pi]\^2\ \[Alpha]1\)\/\(3\ t\^2\ \
\[Psi]\^4\) + \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(15\ \[Psi]\^4\) + 
          2\/\(t\^3\ \[Psi]\^3\) + \(4\ \[Pi]\^2\ \[Alpha]1\)\/\(t\ \[Psi]\^3\
\) + 5\/\(t\^2\ \[Psi]\^2\) + \(2\ \[Pi]\^2\ \[Alpha]1\)\/\[Psi]\^2 + 
          4\/\(t\ \[Psi]\), \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(30\ t\^5\ \[Psi]\^9\
\) + \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(120\ t\^4\ \[Psi]\^8\) + \(\[Pi]\^2\ \
\[Alpha]1\)\/\(6\ t\^5\ \[Psi]\^7\) + \(\[Pi]\^2\ \[Alpha]1\)\/\(12\ t\^4\ \
\[Psi]\^6\) + 1\/\(2\ t\^5\ \[Psi]\^5\) + 19\/\(24\ t\^4\ \[Psi]\^4\) + 
          Log[1\/t]\/\(2\ t\^4\ \[Psi]\^4\) - 
          Log[2\ \[Psi]]\/\(2\ t\^4\ \[Psi]\^4\), \(-\(\(7\ \[Pi]\^4\ \
\[Alpha]2\)\/\(6\ t\^5\ \[Psi]\^10\)\)\) - \(7\ \[Pi]\^4\ \[Alpha]2\)\/\(30\ \
t\^4\ \[Psi]\^9\) - \(\[Pi]\^2\ \[Alpha]1\)\/\(2\ t\^5\ \[Psi]\^8\) - \(\[Pi]\
\^2\ \[Alpha]1\)\/\(6\ t\^4\ \[Psi]\^7\) - 1\/\(2\ t\^5\ \[Psi]\^6\) - 
          1\/\(2\ t\^4\ \[Psi]\^5\), \(49\ \[Pi]\^4\ \[Alpha]2\)\/\(12\ t\^5\ \
\[Psi]\^11\) + \(49\ \[Pi]\^4\ \[Alpha]2\)\/\(72\ t\^4\ \[Psi]\^10\) + \(7\ \
\[Pi]\^2\ \[Alpha]1\)\/\(6\ t\^5\ \[Psi]\^9\) + \(7\ \[Pi]\^2\ \
\[Alpha]1\)\/\(24\ t\^4\ \[Psi]\^8\) + 7\/\(12\ t\^5\ \[Psi]\^7\) + 
          7\/\(24\ t\^4\ \[Psi]\^6\)}}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Massless pair formulas", "Section"],

Cell[CellGroupData[{

Cell["\<\
The expression for the chemical potential 'nu' in terms of n, g, T:\
\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sqt = Sqrt[729\ n\^2 + 3\ \(g\^2\) \(\[Pi]\^2\) T\^6]\)], "Input"],

Cell[BoxData[
    \(\@\(729\ n\^2 + 3\ g\^2\ \[Pi]\^2\ T\^6\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(cbt = \((\(-27\)\ n + sqt)\)\^\(1/3\)\)], "Input"],

Cell[BoxData[
    \(\((\(-27\)\ n + \@\(729\ n\^2 + 3\ g\^2\ \[Pi]\^2\ \
T\^6\))\)\^\(1/3\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(nu = \(\((g\ \[Pi]\^4/3)\)\^\(1/3\)\) 
          T\^2/cbt - \(\((\(\[Pi]\^2/9\)/g)\)\^\(1/3\)\) cbt\)], "Input"],

Cell[BoxData[
    \(\(g\^\(1/3\)\ \[Pi]\^\(4/3\)\ T\^2\)\/\(3\^\(1/3\)\ \((\(-27\)\ n + \
\@\(729\ n\^2 + 3\ g\^2\ \[Pi]\^2\ T\^6\))\)\^\(1/3\)\) - \((1\/g)\)\^\(1/3\)\
\ \((\[Pi]\/3)\)\^\(2/3\)\ \((\(-27\)\ n + \@\(729\ n\^2 + 3\ g\^2\ \[Pi]\^2\ \
T\^6\))\)\^\(1/3\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Rephrase cbt in terms of a convenient variable \[Alpha]:\
\>", \
"Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cbt2 = 
      3\ \(n\^\(1/
              3\)\) \((\(-1\) + Sqrt[Simplify[\(\((729\ n\^2 + 3\ \(g\^2\) \(\
\[Pi]\^2\) T\^6)\)/729\)/n\^2]])\)\^\(1/3\)\)], "Input"],

Cell[BoxData[
    \(3\ n\^\(1/3\)\ \((\(-1\) + \@\(1 + \(g\^2\ \[Pi]\^2\ T\^6\)\/\(243\ \
n\^2\)\))\)\^\(1/3\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(cbt3 = 
      3\ \(n\^\(1/3\)\) \((\(-1\) + \@\(1 + \[Alpha]\))\)\^\(1/3\)\)], "Input"],

Cell[BoxData[
    \(3\ n\^\(1/3\)\ \((\(-1\) + \@\(1 + \[Alpha]\))\)\^\(1/3\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Alpha]0 = \(g\^2\) \(\[Pi]\^2\) \(T\^6/243\)/n\^2\)], "Input"],

Cell[BoxData[
    \(\(g\^2\ \[Pi]\^2\ T\^6\)\/\(243\ n\^2\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Rewrite 'nu' completely in terms of \[Alpha]\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cbt4 = \((\(-1\) + \@\(1 + \[Alpha]\))\)\^\(1/3\)/\[Alpha]\^\(1/6\)\)], \
"Input"],

Cell[BoxData[
    \(\((\(-1\) + \@\(1 + \[Alpha]\))\)\^\(1/3\)\/\[Alpha]\^\(1/6\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(nu2 = \[Pi]\ T/Sqrt[3] \((1/cbt4 - cbt4)\)\)], "Input"],

Cell[BoxData[
    \(\(\[Pi]\ T\ \((\[Alpha]\^\(1/6\)\/\((\(-1\) + \@\(1 + \
\[Alpha]\))\)\^\(1/3\) - \((\(-1\) + \@\(1 + \[Alpha]\))\)\^\(1/3\)\/\[Alpha]\
\^\(1/6\))\)\)\/\@3\)], "Output"]
}, Open  ]],

Cell["Demonstrate that these are the same:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 1\) /. n \[Rule] 2\) /. T \[Rule] 3], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 1\) /. 
            n \[Rule] 2\) /. T \[Rule] 3]}\)], "Input"],

Cell[BoxData[
    \({1.3081326069695551`, 1.3081326069695556`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 3\) /. n \[Rule] 1\) /. T \[Rule] 2], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 3\) /. 
            n \[Rule] 1\) /. T \[Rule] 2]}\)], "Input"],

Cell[BoxData[
    \({0.49689238445494155`, 0.49689238445493994`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 2\) /. n \[Rule] 3\) /. T \[Rule] 1], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 2\) /. 
            n \[Rule] 3\) /. T \[Rule] 1]}\)], "Input"],

Cell[BoxData[
    \({3.7322794788504523`, 3.7322794788504523`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 1\) /. n \[Rule] 3\) /. T \[Rule] 2], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 1\) /. 
            n \[Rule] 3\) /. T \[Rule] 2]}\)], "Input"],

Cell[BoxData[
    \({3.4551663543514115`, 3.455166354351409`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 3\) /. n \[Rule] 2\) /. T \[Rule] 1], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 3\) /. 
            n \[Rule] 2\) /. T \[Rule] 1]}\)], "Input"],

Cell[BoxData[
    \({2.471109692946823`, 2.4711096929468233`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[\(\(nu /. g \[Rule] 2\) /. n \[Rule] 1\) /. T \[Rule] 3], 
      N[\(\(\(nu2 /. \[Alpha] \[Rule] \[Alpha]0\) /. g \[Rule] 2\) /. 
            n \[Rule] 1\) /. T \[Rule] 3]}\)], "Input"],

Cell[BoxData[
    \({0.33291793064328523`, 0.3329179306432842`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some series expansions:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[nu2\ \(Sqrt[3]/\[Pi]\)/T, {\[Alpha], 0, 3}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(2\^\(1/3\)\/\[Alpha]\^\(1/6\)\), 
        "-", \(\[Alpha]\^\(1/6\)\/2\^\(1/3\)\), 
        "+", \(\[Alpha]\^\(5/6\)\/\(6\ 2\^\(2/3\)\)\), 
        "+", \(\[Alpha]\^\(7/6\)\/\(12\ 2\^\(1/3\)\)\), 
        "-", \(\[Alpha]\^\(11/6\)\/\(18\ 2\^\(2/3\)\)\), 
        "-", \(\(5\ \[Alpha]\^\(13/6\)\)\/\(144\ 2\^\(1/3\)\)\), 
        "+", \(\(77\ \[Alpha]\^\(17/6\)\)\/\(2592\ 2\^\(2/3\)\)\), "+", 
        InterpretationBox[\(O[\[Alpha]]\^\(19/6\)\),
          SeriesData[ \[Alpha], 0, {}, -1, 19, 6],
          Editable->False]}],
      SeriesData[ \[Alpha], 0, {
        Power[ 2, 
          Rational[ 1, 3]], 0, 
        Times[ -1, 
          Power[ 2, 
            Rational[ -1, 3]]], 0, 0, 0, 
        Times[ 
          Rational[ 1, 6], 
          Power[ 2, 
            Rational[ -2, 3]]], 0, 
        Times[ 
          Rational[ 1, 12], 
          Power[ 2, 
            Rational[ -1, 3]]], 0, 0, 0, 
        Times[ 
          Rational[ -1, 18], 
          Power[ 2, 
            Rational[ -2, 3]]], 0, 
        Times[ 
          Rational[ -5, 144], 
          Power[ 2, 
            Rational[ -1, 3]]], 0, 0, 0, 
        Times[ 
          Rational[ 77, 2592], 
          Power[ 2, 
            Rational[ -2, 3]]]}, -1, 19, 6],
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[nu2\ \(Sqrt[3]/\[Pi]\)/T, {\[Alpha], \[Infinity], 3}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(\(2\ \@\(1\/\[Alpha]\)\)\/3\), 
        "-", \(8\/81\ \((1\/\[Alpha])\)\^\(3/2\)\), 
        "+", \(32\/729\ \((1\/\[Alpha])\)\^\(5/2\)\), "+", 
        InterpretationBox[\(O[1\/\[Alpha]]\^\(7/2\)\),
          SeriesData[ \[Alpha], 
            DirectedInfinity[ 1], {}, 3, 21, 6],
          Editable->False]}],
      SeriesData[ \[Alpha], 
        DirectedInfinity[ 1], {
        Rational[ 2, 3], 0, 0, 0, 0, 0, 
        Rational[ -8, 81], 0, 0, 0, 0, 0, 
        Rational[ 32, 729]}, 3, 21, 6],
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1400}, {0, 1050}},
ScreenStyleEnvironment->"Presentation",
WindowSize->{1345, 953},
WindowMargins->{{9, Automatic}, {Automatic, 7}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "steiner", \
"src", "spr05", "hal", "doc", "hal", "extras"}, "fermion.ps", \
CharacterEncoding -> "iso8859-1"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 64, 0, 75, "Section"],
Cell[1843, 55, 125, 2, 37, "Input"],

Cell[CellGroupData[{
Cell[1993, 61, 60, 0, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[2078, 65, 118, 2, 43, "Input"],
Cell[2199, 69, 120, 2, 69, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2356, 76, 115, 2, 37, "Input"],
Cell[2474, 80, 220, 3, 68, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2731, 88, 101, 2, 37, "Input"],
Cell[2835, 92, 344, 5, 67, "Output"]
}, Open  ]],
Cell[3194, 100, 87, 3, 44, "Text"],

Cell[CellGroupData[{
Cell[3306, 107, 119, 2, 44, "Input"],
Cell[3428, 111, 137, 2, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3602, 118, 105, 2, 37, "Input"],
Cell[3710, 122, 348, 5, 87, "Output"]
}, Open  ]],
Cell[4073, 130, 58, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[4156, 134, 64, 1, 37, "Input"],
Cell[4223, 137, 347, 5, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4607, 147, 190, 3, 44, "Input"],
Cell[4800, 152, 342, 5, 87, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5179, 162, 107, 2, 37, "Input"],
Cell[5289, 166, 388, 6, 125, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5714, 177, 221, 3, 44, "Input"],
Cell[5938, 182, 385, 6, 87, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6372, 194, 67, 0, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[6464, 198, 107, 2, 43, "Input"],
Cell[6574, 202, 107, 2, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6718, 209, 77, 1, 37, "Input"],
Cell[6798, 212, 1049, 33, 55, "Output"]
}, Open  ]],
Cell[7862, 248, 46, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[7933, 252, 149, 2, 37, "Input"],
Cell[8085, 256, 185, 3, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8307, 264, 85, 1, 37, "Input"],
Cell[8395, 267, 426, 6, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8858, 278, 103, 2, 43, "Input"],
Cell[8964, 282, 111, 2, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9112, 289, 73, 1, 37, "Input"],
Cell[9188, 292, 463, 7, 115, "Output"]
}, Open  ]],
Cell[9666, 302, 56, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[9747, 306, 64, 1, 37, "Input"],
Cell[9814, 309, 411, 6, 122, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10262, 320, 104, 2, 43, "Input"],
Cell[10369, 324, 111, 2, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10517, 331, 85, 1, 37, "Input"],
Cell[10605, 334, 463, 7, 115, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11105, 346, 107, 2, 37, "Input"],
Cell[11215, 350, 579, 8, 170, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11831, 363, 103, 2, 43, "Input"],
Cell[11937, 367, 111, 2, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12085, 374, 112, 2, 37, "Input"],
Cell[12200, 378, 726, 10, 172, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12975, 394, 55, 0, 56, "Subsubsection"],
Cell[13033, 396, 38, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[13096, 400, 243, 6, 44, "Input"],
Cell[13342, 408, 194, 3, 109, "Output"]
}, Open  ]],
Cell[13551, 414, 178, 5, 49, "Text"],

Cell[CellGroupData[{
Cell[13754, 423, 70, 1, 44, "Input"],
Cell[13827, 426, 74, 1, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13938, 432, 70, 1, 37, "Input"],
Cell[14011, 435, 60, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14108, 441, 288, 6, 44, "Input"],
Cell[14399, 449, 164, 2, 82, "Output"]
}, Open  ]],
Cell[14578, 454, 50, 0, 44, "Text"],
Cell[14631, 456, 48, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[14704, 460, 139, 4, 52, "Input"],
Cell[14846, 466, 69, 1, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14952, 472, 82, 1, 37, "Input"],
Cell[15037, 475, 35, 1, 36, "Output"]
}, Open  ]],
Cell[15087, 479, 238, 5, 68, "Text"],

Cell[CellGroupData[{
Cell[15350, 488, 463, 11, 58, "Input"],
Cell[15816, 501, 1045, 23, 66, "Output"]
}, Open  ]],
Cell[16876, 527, 147, 4, 68, "Text"],

Cell[CellGroupData[{
Cell[17048, 535, 310, 6, 71, "Input"],
Cell[17361, 543, 720, 11, 157, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18130, 560, 76, 0, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[18231, 564, 121, 3, 37, "Input"],
Cell[18355, 569, 602, 9, 116, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18994, 583, 86, 1, 43, "Input"],
Cell[19083, 586, 73, 1, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19193, 592, 73, 1, 37, "Input"],
Cell[19269, 595, 439, 6, 61, "Output"]
}, Open  ]],
Cell[19723, 604, 60, 0, 44, "Text"],
Cell[19786, 606, 56, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[19867, 610, 72, 1, 37, "Input"],
Cell[19942, 613, 585, 9, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20564, 627, 86, 1, 40, "Input"],
Cell[20653, 630, 72, 1, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20762, 636, 85, 1, 37, "Input"],
Cell[20850, 639, 431, 6, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21318, 650, 115, 2, 37, "Input"],
Cell[21436, 654, 798, 11, 116, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22271, 670, 112, 2, 37, "Input"],
Cell[22386, 674, 582, 9, 104, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[23017, 689, 84, 1, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[23126, 694, 144, 4, 37, "Input"],
Cell[23273, 700, 307, 5, 59, "Output"]
}, Open  ]],
Cell[23595, 708, 78, 1, 37, "Input"],

Cell[CellGroupData[{
Cell[23698, 713, 226, 4, 58, "Input"],
Cell[23927, 719, 703, 11, 104, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24667, 735, 62, 1, 43, "Input"],
Cell[24732, 738, 57, 1, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24826, 744, 73, 1, 37, "Input"],
Cell[24902, 747, 891, 13, 120, "Output"]
}, Open  ]],
Cell[25808, 763, 56, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[25889, 767, 72, 1, 37, "Input"],
Cell[25964, 770, 611, 9, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26612, 784, 62, 1, 43, "Input"],
Cell[26677, 787, 56, 1, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26770, 793, 85, 1, 37, "Input"],
Cell[26858, 796, 692, 10, 109, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27587, 811, 115, 2, 37, "Input"],
Cell[27705, 815, 1147, 18, 159, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28889, 838, 62, 1, 43, "Input"],
Cell[28954, 841, 56, 1, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29047, 847, 77, 1, 37, "Input"],
Cell[29127, 850, 1339, 19, 171, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[30527, 876, 41, 0, 75, "Section"],

Cell[CellGroupData[{
Cell[30593, 880, 102, 3, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[30720, 887, 86, 1, 44, "Input"],
Cell[30809, 890, 75, 1, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30921, 896, 70, 1, 40, "Input"],
Cell[30994, 899, 105, 2, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31136, 906, 132, 2, 45, "Input"],
Cell[31271, 910, 281, 4, 77, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[31601, 920, 91, 3, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[31717, 927, 182, 4, 45, "Input"],
Cell[31902, 933, 124, 2, 73, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[32063, 940, 107, 2, 58, "Input"],
Cell[32173, 944, 91, 1, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[32301, 950, 84, 1, 43, "Input"],
Cell[32388, 953, 73, 1, 59, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[32510, 960, 77, 2, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[32612, 966, 102, 2, 58, "Input"],
Cell[32717, 970, 97, 2, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[32851, 977, 75, 1, 37, "Input"],
Cell[32929, 980, 188, 3, 85, "Output"]
}, Open  ]],
Cell[33132, 986, 52, 0, 44, "Text"],

Cell[CellGroupData[{
Cell[33209, 990, 208, 3, 37, "Input"],
Cell[33420, 995, 76, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33533, 1001, 208, 3, 37, "Input"],
Cell[33744, 1006, 78, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33859, 1012, 208, 3, 37, "Input"],
Cell[34070, 1017, 76, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34183, 1023, 208, 3, 37, "Input"],
Cell[34394, 1028, 75, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34506, 1034, 208, 3, 37, "Input"],
Cell[34717, 1039, 75, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34829, 1045, 208, 3, 37, "Input"],
Cell[35040, 1050, 77, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[35166, 1057, 48, 0, 56, "Subsubsection"],

Cell[CellGroupData[{
Cell[35239, 1061, 83, 1, 37, "Input"],
Cell[35325, 1064, 1317, 38, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36679, 1107, 93, 1, 37, "Input"],
Cell[36775, 1110, 584, 14, 87, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

